// LeastSq.cpp:   
// 2008-01-08
//
#include <stdlib.h>
//#include <stdio.h>
#include <math.h>

#include "vUtil.h"
#include "dMatrix.h"		//  
#include "LeastSq.h"		//   

//-----------------------------------------------------------------------------------
// implementation LeastSq
//-----------------------------------------------------------------------------------
////////////////////////////////////////////////////
// constructors/destructors
////////////////////////////////////////////////////
//   (sz+1)*sz.
LeastSq::LeastSq(int sz)
{
	a.setSize(sz+1, sz);		// sz , sz+1 
	sol.setSize(sz);
	this->Clear();
	//cnt = 0;
	//sum2= 0;
}
LeastSq::LeastSq(LeastSq &a)
{
	vError("LeastSq::LeastSq(LeastSq &a): copy constructor illegal");
}
LeastSq::~LeastSq() { }
//-----------------------------------------------------------------------------------
void LeastSq::Clear() // 
{
	a.clear(); 
	sol.clear();
	cnt=0; 
	sum2=0;
}	
//-----------------------------------------------------------------------------------
void LeastSq::setSize(int sz) //  
{
	a.setSize(sz + 1,sz);
	sol.setSize(sz);
	Clear(); 
};	
//-----------------------------------------------------------------------------------
int LeastSq::size()	//  
{
	if( a.Mx() != a.My()+1 ) vError("LeastSq::getSize: %d : %d", a.Mx(), a.My() );
	return a.My();
}
//-----------------------------------------------------------------------------------
void LeastSq::add(double *fi, double fx)
//
// fi.get(i) -    f(i,x);
// fx        -    f(x);
//
//          n-1
// a[m,j] = SUM f(j,X[i]*f(m,X[i])
//          i=0
{
	int sz=size();
	int x,y;
	for( y=0; y<sz; y++ )
	{
		for( x=0; x<sz; x++ )
			a(x,y) += fi[x]*fi[y];
		a(sz,y) += fx*fi[y];
	}
	this->sum2 += fx*fx;
	this->cnt  ++;
}
//-----------------------------------------------------------------------------------
void LeastSq::add1(double f0, double fx)	//   dim=1
{
	if (size() != 1) vError("LeastSq::add1: illegal size");
	double LeastSq_add_fi[1];
	LeastSq_add_fi[0]=f0;
	this->add(LeastSq_add_fi, fx);
}
void LeastSq::add2(double f0, double f1, double fx)	//   dim=2
{
	if (size() != 2) vError("LeastSq::add1: illegal size");
	double LeastSq_add_fi[2];
	LeastSq_add_fi[0]=f0;
	LeastSq_add_fi[1]=f1;
	this->add(LeastSq_add_fi, fx);
}
void LeastSq::add3(double f0, double f1, double f2, double fx)	//   dim=3
{
	if (size() != 3) vError("LeastSq::add1: illegal size");
	double LeastSq_add_fi[3];
	LeastSq_add_fi[0]=f0;
	LeastSq_add_fi[1]=f1;
	LeastSq_add_fi[2]=f2;
	this->add(LeastSq_add_fi, fx);
}
void LeastSq::add4(double f0, double f1, double f2, double f3, double fx)	//   dim=4
{
	if (size() != 4) vError("LeastSq::add1: illegal size");
	double LeastSq_add_fi[4];
	LeastSq_add_fi[0]=f0;
	LeastSq_add_fi[1]=f1;
	LeastSq_add_fi[2]=f2;
	LeastSq_add_fi[3]=f3;
	this->add(LeastSq_add_fi, fx);
}
void LeastSq::add5(double f0, double f1, double f2, double f3, double f4, double fx)	//   dim=5
{
	if (size() != 5) vError("LeastSq::add1: illegal size");
	double LeastSq_add_fi[5];
	LeastSq_add_fi[0]=f0;
	LeastSq_add_fi[1]=f1;
	LeastSq_add_fi[2]=f2;
	LeastSq_add_fi[3]=f3;
	LeastSq_add_fi[4]=f4;
	this->add(LeastSq_add_fi, fx);
}
//-----------------------------------------------------------------------------------
//   ,     sol
//    (solve=0)    (solve=2),   sol    -1
//  .. 
double LeastSq::solve() //     sol.  ..
{
	//this->a->saveText("a0.txt", "a0", "%16.2f");
	int res = a.solve2(sol);
	//this->a->saveText("a1.txt", "a1", "%16.2f");
	if( res==0 ) return this->MSE(); 
	return -1;
}
//-----------------------------------------------------------------------------------
double LeastSq::MSE(dvect &fi) // ..   fi
{
	if( this->cnt==0 ) vError("LeastSq::MSE: cnt=0");
	int sz=size();
	int x,y;
	double e = this->sum2;

	for( y=0; y<sz; y++)
	for( x=0; x<sz; x++) 
		e += fi[x]*fi[y]*a(x,y);

	for( y=0; y<sz; y++)
		e -= 2*fi[y]*a(sz,y);

	if( e<-2e-6 ) vError("LeastSq::MSE: e=%f", e);
	if( e<0 ) e=0;

	e = sqrt(e/this->cnt);
	return e;
}
//-----------------------------------------------------------------------------------
double LeastSq::MSE() // ..   sol
{
	return this->MSE(this->sol);
}
//-----------------------------------------------------------------------------------
void LeastSq::append( LeastSq &b )			// this +=    b
{
	a.add(b.a);
	cnt  += b.cnt;
	sum2 += b.sum2;
}
