// dMatrix.cpp: matrix of ldouble
// 2006-11-05

//  ortholin    http://prografix.narod.ru::source.zip::Mathem.cpp(h)::ortholin

//
#pragma warning(disable : 4996)
#ifdef __MINGW32__
  #define __USE_MINGW_ANSI_STDIO 1
#endif
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <memory.h>
#include <stdarg.h>

//typedef long double ldouble;  // in vUtil.h

#include "vUtil.h"
#include "dMatrix.h"		//  

#define dMatrixTextIndexes	//    

//-----------------------------------------------------------------------------
// constructors/destructors
//-----------------------------------------------------------------------------
//   mx.
dvect::dvect(int mx_)
{
	v=NULL;
	mx=mx_;
	if (mx_ > 0) {
		v = new ldouble[mx];
		memset(v, 0, mx*sizeof(ldouble));
	}
}

dvect::dvect( dvect &a ) {vError( "dvect:: copy constructor are illegal");}

dvect::~dvect() { this->release(); }

void	dvect::release()
{
	if( mx<=0 ) return;
	if (v != NULL) delete []v;
	v = NULL;
	mx=0;
}
//-----------------------------------------------------------------------------
void	dvect::clear()		//   
{
	if(mx>0)
	{
		memset(v,0,mx*sizeof(*v));
	}
}
//-----------------------------------------------------------------------------
void	dvect::setSize( int mx_) //    ,   
{
	if( mx==mx_ ) return; //    ,   
	if( mx_<0 ) vError( "dvect::setSize: mx=%d", mx);
	if( v != NULL )
	{
		delete [] v;
		v=NULL;
	}
	mx = mx_;
	if(mx>0)
	{
		v = new ldouble[mx];
		memset(v,0,mx*sizeof(ldouble));
	}
	else { v = NULL; }
}

//-----------------------------------------------------------------------------
void	dvect::setSizeClear( int mx_) //    
{
	if( mx_<0) vError( "dvect::setSize: mx=%d", mx );
	if( mx==mx_ ) //    , 
	{
		clear();
		return;
	}
	if( v != NULL )
	{
		delete [] v;
		v=NULL;
	}
	mx = mx_;
	if(mx>0)
	{
		v = new ldouble[mx];
		//int *p=v; for( int i=0; i<size; i++ ) *p++=0;
		memset(v,0,mx*sizeof(*v));
	}
	else { v = NULL; }
}

//-----------------------------------------------------------------------------
void	dvect::copyFrom(dvect &a)
{
	a.test();
	this->setSize( a.Mx());
	//for( int i=0; i<a.getSize(); i++ ) v[i] = a.v[i];
	memcpy(v, a.v, a.Mx()*sizeof(*v));
}
void	dvect::copyFrom(int N, ldouble *x)
{
	this->setSize(N);
	memcpy(v, x, N*sizeof(*v));
}

//-----------------------------------------------------------------------------
void dvect::test()
{
	if( mx<=0 ) vError( "dvect test: mx=%d", mx );
	if((mx>0) && (v==NULL) ) vError( "dvect: sizes>0, but v==NULL" );
	if( mx>10000) vError( "Cdmatr::test mx=%d", mx);
}

void	dvect::testSameSize( dvect &a  )
{
	test();	a.test();
	if( mx!=a.mx)
		vError( "dvect::testSameSize: mx=%d, a.mx=%d",
		                                 mx,    a.mx);
}

void	dvect::testIndexes(int x) // 0<=x<max
{
	if(x<0 || x>=mx) vError("dvect::testIndexes x=%d",x);
}

void	dvect::normaIndexes(int &x)	//     0<=x<mx
{
	if( x<  0) x=0;
	if( x>=mx) x=mx-1;
}
//-----------------------------------------------------------------------------
ldouble &dvect::operator() (int x) 
{
	if (x<0 || x >= mx) vError("dvect::testIndexes x=%d", x);
	return v[x];
}
ldouble &dvect::operator[] (int x)
{
	if (x<0 || x >= mx) vError("dvect::testIndexes x=%d", x);
	return v[x];
}
//-----------------------------------------------------------------------------
// in/out
void	dvect::saveText(const char *fname, const char *msg, const char *format)
{
	this->test();
	FILE *f = fopen(fname, "w");
	if (!f) vError("saveText: file not open");
	//fprintf( f, "%4d %s\n", mx, msg );
	fprintf(f, "%4d %-12s", mx, msg);
	for (int x = 0; x<mx; x++) fprintf(f, format, v[x]);
	fprintf(f, "\n");
	fclose(f);
}


void	dvect::appendText(const char *fname, const char *msg, const char *format)
{
	this->test();
	FILE *f = fopen(fname, "a");
	if (!f) {
		f = fopen(fname, "w");
		if (!f) vError("saveText: file not open");
	}
	//fprintf( f, "\n%4d %s\n", mx, msg );
	fprintf(f, "%4d %-12s", mx, msg);
	for (int x = 0; x<mx; x++) fprintf(f, format, v[x]);
	fprintf(f, "\n");
	fclose(f);
}

void	dvect::loadText(const char *fname)
{
	this->release();
	FILE *f = fopen(fname, "r");
	if (!f) vError("dvect::loadText: file %s not open", fname);
	char s[255]; int mx_;
	fscanf(f, "%d", &mx_);
	if (mx_ <= 0) vError("dvect::loadText: mx=%d", mx);
	fgets(s, 255, f);
	//if( mx<my ) { mx += my; my = mx-my; mx -= my; }
	this->setSize(mx_);
	double t;
	for (int x = 0; x<mx; x++)
	{
		fscanf(f, "%lf", &t);
		v[x] = t;
	}
	fclose(f);
}
//-----------------------------------------------------------------------------
void	dvect::inc( int x, ldouble value )  // w[x] += value
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(x);
#endif
	v[x] += value;
}

void	dvect::set( ldouble value )
{
	ldouble *p=v;
	//for( int i=mx*my; i>0; i-- ) *p++=value;
	for( int i=mx; i--; ) *p++=value;
}
//-----------------------------------------------------------------------------
ldouble  dvect::minV( int &x )						//   , x -  
{
	int i, im;
	ldouble sm;
	im=0;
	sm=this->v[0];
	for( i=1; i<mx; i++)
		if( v[i]<sm ){ sm=v[i]; im=i; }
	x = im;
	return sm;
}
ldouble  dvect::maxV( int &x )						//   , x -  
{
	int i, im;
	ldouble sm;
	im=0;
	sm=this->v[0];
	for( i=1; i<mx; i++)
		if( v[i]>sm ){ sm=v[i]; im=i; }
	x = im;
	return sm;
}
ldouble dvect::sum()								//  
{
	ldouble sm=0;
	for (int i = 0; i < mx; i++) sm += v[i];
	return sm;
}


//-----------------------------------------------------------------------------
// algebra
//-----------------------------------------------------------------------------
void dvect::add ( dvect &v1, ldouble z)		// this += z*v1
{
	this->testSameSize(v1);
	ldouble *p=v;
	ldouble *p1=v1.v;
	for( int x=0; x<mx; x++ ) *p++ += z*(*p1++);
}
void dvect::add ( ldouble *v1, ldouble z)		// this += z*v1
{
	ldouble *p=v;
	for( int x=0; x<mx; x++ ) *p++ += z*(*v1++);
}
void dvect::add(ldouble z)					//   += z
{
	test();
	ldouble *p = v;
	for (int x = 0; x<mx; x++) *p++ += z;
}
void dvect::mult(ldouble z)					//     v
{
	test();
	ldouble *p = v;
	for (int x = 0; x<mx; x++) *p++ *= z;
}

//#define eps (1e-12)		//  0
void dvect::killZero() //    
{
	int i;
	ldouble eps = 0;
	ldouble *p=v;
	for( i=mx; i--; ) eps += fabs(*p++);
	eps *= macheps;		// eps -  0
	for( i=mx; i--; )
	{
		if( fabs(*p)<eps ) *p=0;
		p++;
	}
}

//-----------------------------------------------------------------------------
void dvect::normalize() //     =1
{
	int i;
	ldouble s=0;
	for( i=0; i<mx; i++ ) s+= dSquare(v[i]);
    if( s==0 ) return;
    s = 1/sqrt(s);
	for( i=0; i<mx; i++ ) v[i] *= s;
}
//-----------------------------------------------------------------------------
ldouble	dvect::length()      // 
{
	ldouble s = 0;
	for (int i = 0; i<mx; i++) s += dSquare(v[i]);
	return sqrt(s);
}
//-----------------------------------------------------------------------------
ldouble	dvect::length2()      //  
{
	ldouble s = 0;
	for (int i = 0; i<mx; i++) s += dSquare(v[i]);
	return s;
}
//-----------------------------------------------------------------------------

ldouble	dvect::dist(dvect &vv)      //   
{
	if( vv.Mx()!=mx ) vError("dvect::dist2 different sizes");
	ldouble s=0;
	for( int i=0; i<mx; i++ ) s+= dSquare(v[i]-vv.v[i]);
	return sqrt(s);
}
ldouble	dvect::dist2(dvect &vv)     //    
{
	if( vv.Mx()!=mx ) vError("dvect::dist2 different sizes");
	ldouble s=0;
	for( int i=0; i<mx; i++ ) s+= dSquare(v[i]-vv.v[i]);
	return s;
}
ldouble	dvect::scProd(dvect &vv)    		        //  
{
	if( vv.Mx()!=mx ) vError("dvect::scProc different sizes");
	ldouble s=0;
	for( int i=0; i<mx; i++ ) s+= v[i]*vv.v[i];
	return s;
}
//-----------------------------------------------------------------------------
void	dvect::setA(int mx_, ...)				//    
{
	this->setSizeClear(mx_);
	ldouble vt;
	va_list marker;

	va_start( marker, mx_ );		// Initialize variable arguments.
	for( int i=0; i<mx_; i++ )
	{
	 	vt = va_arg( marker, ldouble);
		v[i] = vt;
	}
	va_end( marker );              // Reset variable arguments.
}
//-----------------------------------------------------------------------------
void	dvect::setRandom(ldouble mi, ldouble ma)	//     
{
	ldouble co = (ma-mi);
	for( int i=0; i<mx; i++ )
		v[i]=mi + hRand()*co;
}
//-----------------------------------------------------------------------------
void	dvect::append(ldouble z)		//     
{
	ldouble *tmp = v;
	v = new ldouble[mx+1];
	for( int i=0; i<mx; i++ ) v[i]=tmp[i];
	v[mx]=0;
	mx++;
	delete [] tmp;
}
//-----------------------------------------------------------------------------
// sort
int double_compare( const void *arg1, const void *arg2 )
{
	ldouble *d1 = (ldouble*)arg1;
	ldouble *d2 = (ldouble*)arg2;
	// Compare all of both strings:
   return *d1 < *d2;
}

void	dvect::sort()				//   
{
	qsort( (void *)v, (size_t)mx, sizeof(ldouble), double_compare );
}
//-----------------------------------------------------------------------------
////////////////////////////////////////////////////
// constructors/destructors
////////////////////////////////////////////////////
//   mx*my.
dmatr::dmatr(int mx_, int my_)
{
	v = NULL; mx = 0; my = 0; this->setSize(mx_, my_); this->clear(); eps = 1e-10;
	// lambda = 1e-10; 
}

dmatr::dmatr(dmatr &a) { vError("dmatr:: copy constructor are illegal"); }

dmatr::~dmatr() { this->release(); }

void	dmatr::release()
{
	if (mx <= 0 || my <= 0) return;
	if (v != NULL) delete[]v;
	v = NULL;
	mx = my = 0;
}
//////////////////////////////////////////////////////////////////////
void	dmatr::clear()		//   
{
	int size = mx*my;
	if (size>0)
	{
		//int *p=v; for( int i=0; i<size; i++ ) *p++=0;
		memset(v, 0, size * sizeof(*v));
	}
}
//////////////////////////////////////////////////////////////////////
void	dmatr::setSize(int mx_, int my_) //    ,   
{
	if (mx_<0 || my_<0) vError("dmatr::setSize: mx=%d, my=%d", mx, my);
	if (mx == mx_ && my == my_) return; //    ,   
	if (v != NULL)
	{
		delete[] v;
		v = NULL;
	}
	mx = mx_;
	my = my_;
	int size = mx*my;
	if (size>0)
	{
		v = new ldouble[size];
		//int *p=v; for( int i=0; i<size; i++ ) *p++=0;
		memset(v, 0, size * sizeof(*v));
	}
	else { v = NULL; }
}

//////////////////////////////////////////////////////////////////////
void	dmatr::setSizeClear(int mx_, int my_) //    
{
	if (mx_<0 || my_<0) vError("dmatr::setSize: mx=%d, my=%d", mx, my);
	if (mx == mx_ && my == my_) //    , 
	{
		clear();
		return;
	}
	if (v != NULL)
	{
		delete[] v;
		v = NULL;
	}
	mx = mx_;
	my = my_;
	int size = mx*my;
	if (size>0)
	{
		v = new ldouble[size];
		//int *p=v; for( int i=0; i<size; i++ ) *p++=0;
		memset(v, 0, size * sizeof(*v));
	}
	else { v = NULL; }
}

//////////////////////////////////////////////////////////////////////
void	dmatr::copyFrom(dmatr &a)
{
	a.test();
	this->setSize(a.Mx(), a.My());
	//for( int i=0; i<a.getSize(); i++ ) v[i] = a.v[i];
	memcpy(v, a.v, a.getSize() * sizeof(*v));
	//lambda = a.lambda;
	eps = a.eps;
}
//////////////////////////////////////////////////////////////////////
ldouble dmatr::setEps() //      eps ( 0)
{
	eps = 0;
	for (int i = 0; i<mx*my; i++) eps += fabs(v[i]);
	if (eps == 0) eps = macheps;
	else		 eps *= macheps;
	return eps;
}
//////////////////////////////////////////////////////////////////////
void dmatr::test()
{
	if (mx <= 0 || my <= 0) vError("dmatr test: mx=%d, my=%d", mx, my);
	if ((mx>0 || my>0) && (v == NULL)) vError("dmatr: sizes>0, but v==NULL");
	if (mx>200000 || my>200000) vError("Cdmatr::test mx=%d, my=%d", mx, my);
}

void	dmatr::testSameSize(dmatr &a)
{
	test();	a.test();
	if (mx != a.mx || my != a.my)
		vError("dmatr::testSameSize: mx=%d, a.mx=%d, my=%d, a.my=%d",
			mx, a.mx, my, a.my);
}

void	dmatr::testIndexes(int x, int y) // 0<=x,y<max
{
	if (x<0 || x >= mx || y<0 || y >= my) vError("dmatr::testIndexes x=%d, y=%d", x, y);
}

void	dmatr::normaIndexes(int &x, int &y)	//     0<=x,y<max
{
	if (x<  0) x = 0;
	if (x >= mx) x = mx - 1;
	if (y<  0) y = 0;
	if (y >= my) y = my - 1;
}

ldouble *dmatr::getElement(int x, int y)
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(x, y);
#endif
	return v + y*mx + x;
}

ldouble &dmatr::operator() (int x, int y)				//     (  )
{
	if (x<0 || x >= mx) vError("dvect::testIndexes x=%d", x);
	return v[y*mx+x];
}


///////////////////////////////////////////////////////////
// get/put
ldouble dmatr::get(int x, int y)
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(x, y);
#endif
	return v[y*mx + x];
}

ldouble	dmatr::getLast(int y)	//   
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(0, y);
#endif
	return v[y*mx + mx - 1];
}

void	dmatr::put(int x, int y, ldouble value)
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(x, y);
#endif
	v[y*mx + x] = value;
}

void	dmatr::safePut(int x, int y, ldouble value)
{
	normaIndexes(x, y);
	v[y*mx + x] = value;
}

void	dmatr::inc(int x, int y, ldouble value)  // w[x,y] += value
{
#ifdef	dMatrixTextIndexes	//  
	this->testIndexes(x, y);
#endif
	v[y*mx + x] += value;
}

void	dmatr::set(ldouble value)
{
	ldouble *p = v;
	//for( int i=mx*my; i>0; i-- ) *p++=value;
	for (int i = mx*my; i--; ) *p++ = value;
}
///////////////////////////////////////////////////////////////////////////////////
void	dmatr::setRandom(ldouble mi, ldouble ma)	//     
{
	ldouble co = (ma - mi) / RAND_MAX;
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
			put(x, y, mi + rand()*co);
}
////////////////////////////////////////////////////////////////////////////
void    dmatr::getLine(int y, dvect &v)		// v:= line(y)
{
	if (y<0 || y >= my) vError("dmatr::getLine: y=%d, my=%d", y, my);
	v.setSize(mx);
	for (int x = 0; x<mx; x++) v(x) = get(x, y);
}
void    dmatr::getColumn(int x, dvect &v)		// v:= column(x)
{
	if (x<0 || x >= mx) vError("dmatr::getColumn: x=%d, mx=%d", x, mx);
	v.setSize(my);
	for (int y = 0; y<my; y++) v(y) = get(x, y);
}

////////////////////////////////////////////////////////////////////////////
// in/out
void	dmatr::saveText(const char *fname, const char *msg, const char *format)
{
	this->test();
	FILE *f = fopen(fname, "w");
	if (!f) vError("saveText: file not open");
	fprintf(f, "%4d %4d %s\n", mx, my, msg);
	for (int y = 0; y<my; y++)
	{
		for (int x = 0; x<mx; x++) fprintf(f, format, this->get(x, y));
		fprintf(f, "\n");
	}
	fclose(f);
}

static int iSize(ldouble x)
{
	if (x == 0) return 1;
	if (x < 0) return 1 + iSize(-x);
	if (x < 10) return 1;
	if (x < 100) return 2;
	if (x < 1000) return 3;
	if (x < 1e4) return 4;
	if (x < 1e5) return 5;
	if (x < 1e6) return 6;
	if (x < 1e7) return 7;
	if (x < 1e8) return 8;
	if (x < 1e9) return 9;
	if (x < 1e10) return 10;
	if (x < 1e11) return 11;
	if (x < 1e12) return 12;
	if (x < 1e13) return 13;
	if (x < 1e14) return 14;
	if (x < 1e15) return 15;
	if (x < 1e16) return 16;
	return 0;

}

bool	dmatr::isMultipl(ldouble b, ldouble eps)	//     b    eps)
{
	for (int i = 0; i < mx*my; i++)
		if (!isMultiple(this->v[i], b, eps)) return false;
	return true;
}

void	dmatr::saveText2(const char *fname, const char *msg)
{
	this->test();
	ldouble ma = this->maximal(), mi = this->minimal();
	char form[64];
	if (this->isMultipl(1, 1e-13)) {
		int sz = imax2(iSize(ma), iSize(mi));
		sprintf(form, "%%%d.0f ", sz);
		this->saveText(fname, msg, form);
		return;
	}
	ldouble h = 0.1, eps = dmax2(fabs(ma), fabs(mi))*1e-13;
	for (int k = 1; k < 15; k++, h /= 10) {
		if (this->isMultipl(h, eps)) {
			int sz = imax2(iSize(ma / h), iSize(mi / h));
			sprintf(form, "%%%d.%df ", sz + 1, k);
			this->saveText(fname, msg, form);
			return;
		}
	}
	ma = fabs(ma); mi = fabs(mi);
	if( mi>ma) ma = mi;
	if( 0.01< ma && ma<100) this->saveText(fname, msg, " %18.15f");
	else					this->saveText(fname, msg, " %23.15e");
}

void dmatr::saveTextI(const char *fname, const char *msg, const char *format) //   
{
	this->test();
	FILE *f = fopen(fname, "w");
	if (!f) vError("saveText: file not open");
	if (msg)
		fprintf(f, "%4d %4d %s\n", mx, my, msg);
	else
		fprintf(f, "%4d %4d\n", mx, my);
	for (int y = 0; y<my; y++)
	{
		for (int x = 0; x<mx; x++)
		{
			fprintf(f, format, x, y, this->get(x, y));
		}
		fprintf(f, "\n");
	}
	fclose(f);
}


void	dmatr::appendText(const char *fname, const char *msg, const char *format)
{
	this->test();
	FILE *f = fopen(fname, "a");
	if (!f) {
		f = fopen(fname, "w");
		if (!f) vError("saveText: file not open");
	}
	fprintf(f, "\n%4d %4d %s\n", mx, my, msg);
	for (int y = 0; y<my; y++)
	{
		for (int x = 0; x<mx; x++) fprintf(f, format, this->get(x, y));
		fprintf(f, "\n");
	}
	fclose(f);
}

void	dmatr::loadText(const char *fname)
{
	this->release();
	FILE *f = fopen(fname, "r");
	if (!f) vError("dmatr::loadText: file %s not open", fname);
	char s[255]; int mx_, my_;
	fscanf(f, "%d%d", &mx_, &my_);
	if (mx_ <= 0 || my_ <= 0) vError("dmatr::loadText: mx=%d, my=%d", mx, my);
	fgets(s, 255, f);
	//if( mx<my ) { mx += my; my = mx-my; mx -= my; }
	this->setSize(mx_, my_);
	ldouble t;
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
		{
			fscanf(f, "%lf", &t);
			this->put(x, y, t);
		}
	fclose(f);
}


//////////////////////////////////////////////////////////////
// algebra
//////////////////////////////////////////////////////////////
#define DBL_MAX_ 1.797693134862315e+308

ldouble	dmatr::minimal(int &x_, int &y_) // minimal value
{
	ldouble vi, min_ = DBL_MAX_;
	int imin;
	for (int i = mx*my - 1; i >= 0; i--)
		if ((vi = v[i])<min_) { imin = i; min_ = vi; }
	y_ = imin / mx; x_ = imin%mx;
	return min_;
}

ldouble	dmatr::minimal()				// minimal value
{
	ldouble vi, min_ = DBL_MAX_;
	for (int i = mx*my - 1; i >= 0; i--)
		if ((vi = v[i])<min_) { min_ = vi; }
	return min_;
}

ldouble	dmatr::maximal(int &x_, int &y_) // maximal value
{
	ldouble vi, max_ = -DBL_MAX_;
	int imax = 0;
	for (int i = mx*my - 1; i >= 0; i--)
		if ((vi = v[i])>max_) { imax = i; max_ = vi; }
	y_ = imax / mx; x_ = imax%mx;
	return max_;
}

ldouble	dmatr::maximal()				// maximal value
{
	ldouble vi, max_ = -DBL_MAX_;
	for (int i = mx*my - 1; i >= 0; i--)
		if ((vi = v[i])>max_) { max_ = vi; }
	return max_;
}

ldouble	dmatr::maxCol(int x, int &y_)				// maximal value in column
{
	ldouble vi, max_ = -DBL_MAX_;
	for (int y = 0; y<my; y++)
		if ((vi = get(x, y))>max_) { max_ = vi; y_ = y; }
	return max_;
}

ldouble	dmatr::minCol(int x, int &y_)				// minimal value in column
{
	ldouble vi, min_ = +DBL_MAX_;
	for (int y = 0; y<my; y++)
		if ((vi = get(x, y))<min_) { min_ = vi; y_ = y; }
	return min_;
}

ldouble	dmatr::maxLine(int &x_, int y)			// maximal value in line
{
	ldouble vi, max_ = -DBL_MAX_;
	for (int x = 0; x<mx; x++)
		if ((vi = get(x, y))>max_) { max_ = vi; x_ = x; }
	return max_;
}



ldouble	dmatr::minLine(int &x_, int y)			// minimal value in line
{
	ldouble vi, min_ = +DBL_MAX_;
	for (int x = 0; x<mx; x++)
		if ((vi = get(x, y))<min_) { min_ = vi; x_ = x; }
	return min_;
}

ldouble	dmatr::sumLine(int y)				// summa values in line
{
	ldouble sum = 0;
	for (int x = 0; x < mx; x++)
		sum += get(x, y);
	return sum;
}

ldouble	dmatr::sumCol(int x)						// summa values in column
{
	ldouble sum = 0;
	for (int y = 0; y < my; y++)
		sum += get(x, y);
	return sum;
}

void	dmatr::copyLine(dmatr &a, int y, int ya)	// y- (this) := ya- ()
{
	if( mx != a.Mx() ) vError("dmatr::copyLine: different line sizes");
	if( y<0 || y>my) vError("dmatr::copyLine: illegal y");
	if( ya<0 || ya>a.My()) vError("dmatr::copyLine: illegal ya");
	vdbl_cop(mx, getElement(0,y), a.getElement(0,ya));
}
void	dmatr::copyCol (dmatr &a, int x, int xa)	// x- (this) := xa- ()
{
	if( my != a.My() ) vError("dmatr::copyCol: different column sizes");
	if( x<0 || x>mx) vError("dmatr::copyCol: illegal x");
	if( xa<0 || xa>a.Mx()) vError("dmatr::copyCol: illegal xa");
	for( int y=0; y<my; y++)
		put(x,y, a(xa,y));
}

void	dmatr::copyBlock(dmatr &a, int x0, int y0, int dx, int dy)// this := (a)  dx*dy
{
	if( dx<=0 || dx>a.Mx()-x0 ) dx = a.Mx()-x0;
	if( dy<=0 || dy>a.My()-y0 ) dy = a.My()-y0;
	setSize(dx, dy);
	for( int y=0; y<dy; y++ )
		for( int x=0; x<dx; x++ )
			put(x,y, a(x0+x, y0+y));
}

void dmatr::add(dmatr &a)					// this += a
{
	testSameSize(a);
	ldouble *p = this->getElement(0, 0);
	ldouble *pa = a.getElement(0, 0);
	for (int x = 0; x<mx*my; x++) *p++ += *pa++;
}

void dmatr::add(ldouble z)						//   += z
{
	test();
	ldouble *p = this->getElement(0, 0);
	for (int x = 0; x<mx*my; x++) *p++ += z;
}

void dmatr::mult(ldouble z)					//     v
{
	test();
	ldouble *p = this->getElement(0, 0);
	for (int x = 0; x<mx*my; x++) *p++ *= z;
}

void dmatr::mult(dvect &dst, dvect &src)	// dst := this*src; src.size    this->mx
{
	test();
	if (src.size() != this->mx) vError("dmatr::mult: src.Size=%d; mx=%d ", src.size(), this->mx);
	dst.setSize(this->my);
	ldouble su;
	for (int y = 0; y<my; y++)
	{
		su = 0;
		for (int x = 0; x<mx; x++) su += this->get(x, y)*src(x);
		dst(y) = su;
	}
}

void dmatr::mult(dmatr &a, dmatr &b)			// this := a*b
{
	if (a.Mx() != b.My()) vError("dmatr::mult: a.Mx=%d; b.My=%d ", a.Mx(), b.My());
	this->setSize(b.Mx(), a.My());
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
		{
			ldouble s = 0;
			//for( int i=0; i<a.Mx(); i++ )
			//	s += a.get(i,y)*b.get(x,i);
			ldouble *ay = a.getElement(0, y);
			ldouble *bx = b.getElement(x, 0);
			for (int i = 0; i<a.Mx(); i++, ay++, bx += b.Mx())
				s += ay[0] * bx[0];
			put(x, y, s);
		}
}
void dmatr::multL(dvect &dst, dvect &src)	// dst := src*this; src.size    this->my
{
	test();
	if (src.size() != this->my) vError("dmatr::multL: src.Size=%d; my=%d ", src.size(), this->my);
	dst.setSize(this->mx);
	ldouble su;
	for (int x = 0; x<mx; x++)
	{
		su = 0;
		for (int y = 0; y<my; y++) su += this->get(x, y)*src(y);
		dst(x) = su;
	}
}

ldouble	dmatr::bilin(dvect &L, dvect &R)			// retrun sum a[i,j]*L[i]*R[j]
{
	test();
	if (L.size() != this->my) vError("dmatr::bilin: L.Size=%d; my=%d ", L.size(), this->my);
	if (R.size() != this->mx) vError("dmatr::bilin: R.Size=%d; mx=%d ", R.size(), this->mx);
	ldouble su = 0;
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
		{
			su += L(y)*this->get(x, y)*R(x);
		}
	return su;
}

void dmatr::multLine(int y, ldouble z)	//    v
{
	test();
	ldouble *p = this->getElement(0, y);
	for (int x = 0; x<mx; x++) *p++ *= z;
}

void dmatr::multCol(int x, ldouble z)	//    v
{
	test();
	ldouble *p = this->getElement(x, 0);
	for (int y = 0; y<my; y++, p += mx) *p *= z;
}

void dmatr::addMult(int y, int y1, ldouble z)//      y1,   v
{
	ldouble *p = this->getElement(0, y);
	ldouble *p1 = this->getElement(0, y1);
	for (int x = 0; x<mx; x++) *p++ += z*(*p1++);
}

void dmatr::addL(int y, dvect &r, ldouble z)	//      r,   z
{
	if (this->Mx() != r.Mx()) vError("dmatr::addL:  this->Mx() != r.Mx()");
	ldouble *p = this->getElement(0, y);
	ldouble *p1 = &r(0);
	for (int x = 0; x<mx; x++) *p++ += z*(*p1++);
}

void dmatr::swapLine(int y, int y1) //   y, y1
{
	if (y == y1) return;
	ldouble *p = this->getElement(0, y);
	ldouble *p1 = this->getElement(0, y1);
	ldouble tmp;
	for (int x = 0; x<mx; x++)
	{
		tmp = *p;
		*p++ = *p1;
		*p1++ = tmp;
	}
}
void dmatr::swapCol(int x, int x1)	//   x,x1
{
	if (x == x1) return;
	ldouble *p = this->getElement(x, 0);
	ldouble *p1 = this->getElement(x1, 0);
	ldouble tmp;
	for (int x = 0; x<mx; x++)
	{
		tmp = *p;
		*p = *p1;   p += mx;
		*p1 = tmp;  p1 += mx;
	}
}
//#define eps (1e-12)		//  0
void dmatr::killZero() //    
{
	ldouble *p = v;
	for (int i = mx*my; i--; )
	{
		if (fabs(*p)<eps) *p = 0;
		p++;
	}
}

void dmatr::killCol(int x, int y) //    [x,y]     
{
	ldouble z = get(x, y);
	if (fabs(z)<eps) vError("dmatr::killCol, v[x,y] almost 0");
	this->multLine(y, 1 / z);	//   v[x,y]=1
	for (int y1 = 0; y1<my; y1++)
		if (y1 != y)
			this->addMult(y1, y, -this->get(x, y1));
}


ldouble dmatr::maxInColDown(int x, int &y) //  w[x,y] -       x   y
{
	ldouble ma = -1e99;
	int    may = -1;
	for (int y1 = y; y1<my; y1++)
	{
		ldouble z = fabs(this->get(x, y1));
		if (z>ma) { ma = z; may = y1; }
	}
	y = may;
	return ma;
}

int dmatr::triangulate()	//    .  
{
	int x, y, y0;
	ldouble s;
	for (x = 0, y = 0; x<mx; x++)	// x -  
	{							// y -   
		this->killZero();
		y0 = y; s = this->maxInColDown(x, y0);
		if (s<eps) continue;	//    .    
								//   x   y (x,y0) -  
		if (y0<0)
			this->saveText("errrr.txt", "error!");
		this->swapLine(y, y0);
		//    x   y (x,y) -  
		this->killCol(x, y); // w[x,y]==1,    =0.
		y++;
		//this->saveText("t.txt", "" );
	}
	return y;
}


////////////////////////////////////////////////////////////////////////////////////////
void	dmatr::addSq(ldouble *u)	// v[k,l]+=u[k]*u[l]
{
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
			v[y*mx + x] += u[x] * u[y];
}

////////////////////////////////////////////////////////////////////////////////////////
void dmatr::normalize() //     =1
{
	int i;
	ldouble s = 0;
	for (i = 0; i<mx*my; i++) s += dSquare(v[i]);
	if (s == 0) return;
	s = 1 / sqrt(s);
	for (i = 0; i<mx*my; i++) v[i] *= s;
}

//---------------------------------------------------------------
// this := transpose(a)*a
//  , w[i,j] := . i-  j-   v
void dmatr::mTransp(dmatr &a)      // this := transpose(a)*a,   my=mx-1
{
	int x, y, k;
	this->setSizeClear(a.Mx(), a.Mx() - 1);
	ldouble norm = 0;								//    
	for (int i = 0; i<mx*my; i++) norm += dSquare(a.v[i]);
	if (norm == 0) return;						//  ,  
	norm = 1.0 / norm;

	ldouble s;
	for (y = 0; y<my; y++)
		for (x = 0; x<mx; x++)
		{
			s = 0;
			for (k = 0; k<a.My(); k++)
				s += a.get(x, k)*a.get(y, k);
			s *= norm;
			this->put(x, y, s);
		}
}
//---------------------------------------------------------------
// this := transpose(a)*a
//  , this[x,y] := . x-  y-   a
void dmatr::mTranspose(dmatr &a)      // this := transpose(a)*a
{
	int x, y;
	this->setSizeClear(a.Mx(), a.Mx());

	ldouble s;
	for (y = 0; y<my; y++)
		for (x = 0; x <= y; x++)
		{
			s = 0;
			//for(k=0; k<a.My(); k++ )
			//    s += a.get(x,k)*a.get(y,k);
			ldouble *ay = a.getElement(y, 0);
			ldouble *bx = a.getElement(x, 0);
			for (int i = 0; i<a.My(); i++, ay += mx, bx += mx)
				s += ay[0] * bx[0];
			this->put(x, y, s);
			if (x<y) this->put(y, x, s);

			//if (x == 0)	printf(" %d   \r", y);
		}

}

///////////////////////////////////////////////////////////////////////////////////
ldouble	dmatr::lineLength(int y)					//  y- 
{
	ldouble s = 0;
	for (int x = 0; x<this->Mx(); x++) s += dSquare(this->get(x, y));
	return(sqrt(s));
}
///////////////////////////////////////////////////////////////////////////////////
ldouble	dmatr::columnLength(int x)				//  x- 
{
	ldouble s = 0;
	for (int y = 0; y<this->My(); y++) s += dSquare(this->get(x, y));
	return(sqrt(s));
}
///////////////////////////////////////////////////////////////////////////////////
ldouble	dmatr::isOrthonormal()	//     
{
	if (mx != my) vError("dmatr::isOrthonormal: mx != my");
	ldouble res = 0;
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
		{
			ldouble t = 0;
			for (int z = 0; z<my; z++)
				t += get(x, z)*get(y, z);
			if (x == y) res += fabs(t - 1);
			else     res += fabs(t);
		}
	return res;
} // ldouble	dmatr::isOrthonormal()	//     
  ///////////////////////////////////////////////////////////////////////////////////
void    dmatr::setFourierBasis(int N)	//    f   -  N
{
	this->setSize(N, N);
	ldouble sqrtN = sqrt((ldouble)N);
	for (int x = 0; x<N; x++)
		for (int u = 0; u<N; u++)
		{
			ldouble C = (u == 0) ? 1 : sqrt(2.0);
			this->put(x, u, C / sqrtN*cos((2 * x + 1)*dPi*u / (2.0*N)));
		}

}
///////////////////////////////////////////////////////////////////////////////////
bool    dmatr::NormaLine(int y)					//    y   1
{
	ldouble s = 0;
	for (int x = 0; x<this->Mx(); x++) s += dSquare(this->get(x, y));
	if (s == 0) return false;
	s = sqrt(s);
	if (s<this->eps) { this->multLine(y, 0); return false; }
	this->multLine(y, 1 / s);
	return true;
}
bool   dmatr::NormaCol(int x)		//    x   1. false -    0
{
	ldouble s = 0;
	for (int y = 0; y<this->My(); y++) s += dSquare(this->get(x, y));
	if (s == 0) return false;
	s = sqrt(s);
	if (s<this->eps) { this->multCol(x, 0); return false; }
	this->multCol(x, 1 / s);
	return true;
}
///////////////////////////////////////////////////////////////////////////////////
ldouble dmatr::ScProd(int y1, int y2)				//   
{
	ldouble s = 0;
	for (int x = 0; x<this->Mx(); x++) s += this->get(x, y1)*this->get(x, y2);
	return s;
}
ldouble dmatr::ScProdC(int x1, int x2)			//   
{
	ldouble s = 0;
	for (int y = 0; y<this->My(); y++) s += this->get(x1, y)*this->get(x2, y);
	return s;
}

///////////////////////////////////////////////////////////////////////////////////
ldouble  dmatr::ScProd(int y, dvect &r)			//   y-    r
{
	if (this->Mx() != r.Mx()) vError("dmatr::ScProd:  this->Mx() != r.Mx()");
	ldouble s = 0;
	for (int x = 0; x<this->Mx(); x++) s += this->get(x, y)*r(x);
	return s;
}
///////////////////////////////////////////////////////////////////////////////////
void    dmatr::GramShmidt()		//        
{
	int z, z1;
	for (z = 0; z<my; z++)
	{
		this->NormaLine(z);
		for (z1 = 0; z1<z; z1++)
		{
			this->addMult(z, z1, -this->ScProd(z, z1));
		}
		this->NormaLine(z);
	}
	this->killZero();
}
///////////////////////////////////////////////////////////////////////////////////
void    dmatr::transpose(dmatr &a)				// this :=transpose(a)
{
	this->setSize(a.My(), a.Mx());
	for (int y = 0; y<my; y++)
		for (int x = 0; x<mx; x++)
		{
			this->put(x, y, a.get(y, x));
		}
}
////////////////////////////////////////////////////////////////////////////////////////
void    dmatr::transpose()				//  " "  
{
	if (mx != my) vError("dmatr::transpose(void): mx != my");
	for (int y = 1; y<my; y++)
		for (int x = 0; x<y; x++)
		{
			ldouble t = get(x, y);
			put(x, y, get(y, x));
			put(y, x, t);
		}
}
////////////////////////////////////////////////////////////////////////////////////////
ldouble dmatr::isSymmetric()	// return difference for symmtricity
{
	if (mx != my) vError("dmatr::isSymmetric: mx != my");
	ldouble res = 0;
	for (int y = 1; y<my; y++)
		for (int x = 0; x<y; x++)
		{
			res = res + fabs(get(x, y) - get(y, x));
		}
	return res;
} // ldouble dmatr::isSymmetric()	// return difference for symmtricity
  ///////////////////////////////////////////////////////////////////////////////////
ldouble d_sign(ldouble *a, ldouble *b)
{
	ldouble x = (*a < 0 ? -*a : *a);
	return(*b < 0 ? -x : x);
}
//-----------------------------------------------------------------------------
void scProdLines(dmatr &a, dmatr &b, dmatr &scProd) // scProd:=    a  b
{
	if (a.Mx() != b.Mx()) vError("scProdLines: a.Mx() != b.Mx()");
	int ma = a.My(), mb = b.My();
	scProd.setSize(ma, mb);
	dvect la, lb;
	for (int ya = 0; ya<ma; ya++)
		for (int yb = 0; yb<mb; yb++)
		{
			a.getLine(ya, la);
			b.getLine(yb, lb);
			scProd.put(ya, yb, la.scProd(lb));
		}
}
//-----------------------------------------------------------------------------
//   :
// 0 -  
// 1 -      
int dmatr::solve(dvect &sol)					// sol :=
{
	this->setEps();				//  eps
	if (mx != my + 1) return 1;
	int rank = this->triangulate();
	if (fabs(this->get(my - 1, my - 1) - 1)> eps) return 1;
	this->getColumn(mx - 1, sol);
	return 0;
}
//-----------------------------------------------------------------------------
ldouble  dmatr::det() //   
{
	if (this->mx != this->my) vError("dmatr::det(): mx=%d, my=%d", mx, my);
	dmatr a;
	a.copyFrom(*this);
	ldouble s, de = 1;
	int y0;
	for (int x = 0; x<mx; x++)	// x -  
	{
		y0 = x;
		s = a.maxInColDown(x, y0);
		if (s<eps) return 0;	//    .  =0
		de *= s;
		//   x   y (x,y0) -  
		if (x != y0) {
			a.swapLine(x, y0);	//  
			de = -de;			//  *= -1
		}
		//    x   y (x,x) -  
		a.killCol(x, x); // w[x,x]==1,    =0.
	}
	return de;
} // ldouble  dmatr::det() //   
//-----------------------------------------------------------------------------
// this := revers(a0)
bool dmatr::revers(dmatr &a0)					// this := revers(a)
{
	int mx = a0.Mx();
	if (mx != a0.My()) vError("dmatr::revers, mx=%d  != my=%d", mx, a0.My());
	dmatr a;
	a.copyFrom(a0);
	this->setSizeClear(mx, mx);
	int x, y;
	for (x = 0; x<mx; x++) this->put(x, x, 1);
	// now this = E
	ldouble s, eps;
	eps = a.setEps();
	for (x = 0; x<mx; x++)
	{
		y = x;
		a.maxInColDown(x, y);
		s = a.get(x, y);
		if (fabs(s)<eps) return false;
		if (y != x) { a.swapLine(x, y); this->swapLine(x, y); }
		a.multLine(x, 1 / s);
		this->multLine(x, 1 / s);
		//  a[x,x]=1
		for (y = 0; y<mx; y++)
		{
			if (y == x) continue;
			s = -a.get(x, y);
			a.addMult(y, x, s);
			this->addMult(y, x, s);
		}
	}
	return true;
}
//-----------------------------------------------------------------------------
void  dmatr::EigenQL(dvect &eval, dmatr &evec)	// eigenvalues (increased) and eigenvectors of symmetric matrix, ev:=eigenvalues(this), v:=eigenvectors(this)
//    evec     (this)!
//         
//                         
//     :
//        n -  
//        a -  .     
//             ,      
//     :
//        d -   
//        v -   
//
//***************************************************************************//
//void jacobi ( const nat n, double * const * a, double * d, double * const * v )
//                                         this        eval                evec
{
	int n = this->Mx();
	if (n== 0) return;
	this->setEps();
	if (isSymmetric() > 100 * eps) vError("dmatr::EigenQL: matrix not symmetric");
	int ierr = 0;
	eval.setSizeClear(n);
	evec.setSizeClear(n, n);
	dvect b(n), z(n);
	int i, j;
	for (i = 0; i < n; ++i) {
		evec(i, i) = 1;
		b[i] = eval[i] = get(i, i);
	}
	for (i = 0; i < 50; ++i)
	{
		double sm = 0.;
		int p, q;
		for (p = 0; p < n - 1; ++p)
		{
			for (q = p + 1; q < n; ++q) sm += fabs(get(p,q));
		}
		if (sm == 0) break;
		const double tresh = i < 3 ? 0.2 * sm / (n*n) : 0.;
		for (p = 0; p < n - 1; ++p)
		{
			for (q = p + 1; q < n; ++q)
			{
				const double g = 1e12 * fabs(get(p,q));
				if (i >= 3 && fabs(eval[p]) > g && fabs(eval[q]) > g) put(p,q, 0);
				else
					if (fabs(get(p,q)) > tresh)
					{
						const double theta = 0.5 * (eval[q] - eval[p]) / get(p,q);
						double t = 1. / (fabs(theta) + sqrt(1. + theta*theta));
						if (theta < 0) t = -t;
						const double c = 1. / sqrt(1. + t*t);
						const double s = t * c;
						const double tau = s / (1. + c);
						const double h = t * get(p,q);
						z[p] -= h;
						z[q] += h;
						eval[p] -= h;
						eval[q] += h;
						put(p,q,0);
						for (j = 0; j < p; ++j)
						{
							const double g = get(j,p);
							const double h = get(j,q);
							put(j, p, g - s * (h + g * tau));
							put(j, q, h + s * (g - h * tau));
						}
						for (j = p + 1; j < q; ++j)
						{
							const double g = get(p, j);
							const double h = get(j, q);
							put(p, j, g - s * (h + g * tau));
							put(j, q, h + s * (g - h * tau));
						}
						for (j = q + 1; j < n; ++j)
						{
							const double g = get(p,j);
							const double h = get(q,j);
							put(p, j, g - s * (h + g * tau));
							put(q, j, h + s * (g - h * tau));
						}
						for (j = 0; j < n; ++j)
						{
							const double g = evec(j,p);
							const double h = evec(j,q);
							evec(j,p) = g - s * (h + g * tau);
							evec(j,q) = h + s * (g - h * tau);
						}
					}
			}
		}
		for (p = 0; p < n; ++p)
		{
			eval[p] = (b[p] += z[p]);
			z[p] = 0.;
		}
	}
	//      
	//printf("n=%4d, i=%3d\n", n, i);


	// eval[i]     -  
	// evec[i][*]  -    (  )
	//     
	for (i = 0; i < n - 1; i++) {
		//     eval[i]..eval[n-1]
		double max = eval[i];
		int jmax = i;
		for(j=i+1; j<n; j++) 
			if (eval[j] > max) {
				max = eval[j];
				jmax = j;
			}
		//     eval[jmax]
		if (jmax != i) {  // 
			swapDbl(eval[i], eval[jmax]);
			evec.swapLine(i, jmax);
		}
	}

	//     .  .
	for (i = 1; i < n; i++)
		for (j = 0; j < i; j++)
			put(j, i, get(i, j));

} // void  dmatr::EigenQL(dvect &eval, dmatr &evec)	// eigenvalues (increased) and eigenvectors of symmetric matrix, ev:=eigenvalues(this), v:=eigenvectors(this)
//-----------------------------------------------------------------------------
int		dmatr::solve2(dvect &sol)					// sol := ,  
{
	dmatr a1; a1.copyFrom(*this);		// 
	int res = a1.solve(sol);
	if (res == 0) return 0;
	//    !
	a1.setSizeClear(mx, mx - 1);
	double sum = 0;						//   
	for (int y1 = 0; y1 < mx - 1; y1++)
		for (int x1 = 0; x1 < mx; x1++) {
			ldouble t = this->ScProdC(x1, y1);
			a1(x1, y1) = t;
			sum += fabs(t);
		}
	sum *= 1e-10;
	//   a1  ,   sum
	for (int y1 = 0; y1 < mx - 1; y1++) a1(y1, y1) += sum;
	return a1.solve(sol);
}
//-----------------------------------------------------------------------------


//
