// dfunc2.cpp  functions of two variables f(x,y)
// by Sergei Khashin 2017-06-06
#include <math.h>
//#include <stdio.h>

#include "vUtil.h"			//   
#include "BiQuadr.h"		//  
#include "dfunc2.h"			// functions of one variable f(x)


double f2min(double fm, double f0, double f1, double &x);

//-----------------------------------------------------------------------------
//   f(x)  f(x) = a0 + a1*x + a2*x*x , :
// f(-1) = fm,
// f( 0) = f0,
// f( 1) = f1.
//      [-1,1],     x,  f(x)
// a0 = f0,
// a1 = (f1-fm)/2,
// a2 = (f1-2*f0+fm)/2 = (f1+fm)/2 - f0,
// x  = (fm-f1)/(2*(f1-2*f0+fm)).
double f2min(double fm, double f0, double f1, double &x)
{
	double a2 = 0.5*(f1 + fm - 2 * f0);
	if (a2 <= 0) {						//  ,   
		if (fm < f1) {
			x = -1;
			return fm;
		}
		else {
			x = 1;
			return f1;
		}
	}
	// now a2 > 0
	double a1 = 0.5*(f1 - fm);
	double a0 = f0;
	x = -0.5*a1 / a2;
	if( x<-1) {
		x = -1;
		return fm;
	}
	if (x>1) {
		x = 1;
		return f1;
	}
	return a0 + a1*x + a2*x*x;
}
//-----------------------------------------------------------------------------
static double xi(double x0, double h, int i) { return x0 + h*i; }
static int  indx(int ix, int iy, int N) { return ix + N*iy; }
//-----------------------------------------------------------------------------
//   
//  : -maxX <= x,y <= maxX,
//       nBlocks*nBlocks 
//  ,    (x,y)  epsX
//      epsF.
// (xy,yr) -  .
//   f(xr,yr).
double f2min2(MathFunc2 &f, double maxX, int nBlocks, double epsX, double epsF, double &xr, double &yr)
{
	int N = nBlocks;
	double *fv = new double[N*N];				//   f(x[i], y[i])
	double h = 2 * maxX / (N - 1);				//   x,y
	double xMin1 = dbl_MAX, yMin1 = dbl_MAX, fMin1 = dbl_MAX;
	for (int iy = 0; iy < N; iy++) {
		double y = xi(-maxX, h, iy);
		for (int ix = 0; ix < N; ix++) {
			double x = xi(-maxX, h, ix);
			// (x,y) -   
			double fxy = f(x, y);
			fv[indx(ix, iy, N)] = fxy;
			if (fxy < fMin1) {
				xMin1 = x;
				yMin1 = y;
				fMin1 = fxy;
			}
		}
	}
	//   .
	//      f(xMin1, yMin1) = fMin1
	//     3*3
	double f9[9];
	for (int iy = 0; iy < N - 2; iy++) {
		//double y = xi(-maxX, h, iy);
		for (int ix = 0; ix < N - 2; ix++) {
			//double x = xi(-maxX, h, ix);
			// (x,y) -   
			int I0 = indx(ix, iy, N);		//    
			f9[0] = fv[I0 + 0]; f9[1] = fv[I0 + 1]; f9[2] = fv[I0 + 2]; I0 += N;
			f9[3] = fv[I0 + 0]; f9[4] = fv[I0 + 1]; f9[5] = fv[I0 + 2]; I0 += N;
			f9[6] = fv[I0 + 0]; f9[7] = fv[I0 + 1]; f9[8] = fv[I0 + 2];
			double ex, ey;
			double ft = minBiSq(f9, ex, ey, 1);
			if (ft < fMin1) {
				xMin1 = xi(-maxX, h, ix) + h*(1 + ex);
				yMin1 = xi(-maxX, h, iy) + h*(1 + ey);
				fMin1 = ft;
			}
		}
	}
	//  (x1,y1) -       
	h *= 0.5;
	double fMin2 = fMin1;
	while (h > epsX) {
		double y;
		y = yMin1 - h; f9[0] = f(xMin1 - h, y); f9[1] = f(xMin1, y); f9[2] = f(xMin1 + h, y);
		y = yMin1    ; f9[3] = f(xMin1 - h, y); f9[4] = f(xMin1, y); f9[5] = f(xMin1 + h, y);
		y = yMin1 + h; f9[6] = f(xMin1 - h, y); f9[7] = f(xMin1, y); f9[8] = f(xMin1 + h, y);
		double ex, ey;
		double ft = minBiSq(f9, ex, ey, 1);
		if (ft < fMin1) {
			xMin1 += h*ex;
			yMin1 += h*ey;
			fMin1 = ft;
		}
		if (fMin1 > fMin2 - epsF) break;
		fMin2 = fMin1;
	}
	delete[]fv;
	xr = xMin1;
	yr = yMin1;
	return fMin1;
}
