// igeom.cpp 2017-01-20    
// :
// :     (x,y);
// :    (x0,y0)-(x1,y1)
// :   8  (x0,y0)-(x1,y1)-(x2,y2)-(x3,y3) 

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#ifdef __GNUC__
#pragma GCC diagnostic ignored "-Wwrite-strings"
#pragma GCC diagnostic ignored "-Wformat"
#endif


#include <math.h>
#include "igeom.h"			//    

const double dPi = 3.14159265358979324;
const int MAXI = 0x7fffffff;
const int MINI = 0x80000000;
#define round(x) ((int)floor((x)+0.5 ))		//   

//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
int sumSq(int a, int b) {
	return a*a + b*b;
}
double hy2(int a, int b) {
	return sqrt((double)(a*a + b*b));
}
//-----------------------------------------------------------------------------
void setMinMax(int x, int &min, int &max) {	//  corrMinMax  vUtil
	if (x < min) min = x;
	if (x > max) max = x;
}
//-----------------------------------------------------------------------------
void neighb4(int x0, int y0, int K, int h, int &x2, int &y2)// K=0..3, (x2,y2):=   K   
{
	x2 = x0; y2 = y0;
	if (K == 0) { x2 += h;  }
	else if (K == 1) { y2 += h; }
	else if (K == 2) { x2 -= h; }
	else { y2 -= h;  }
}
//-----------------------------------------------------------------------------
void neighb8(int x0, int y0, int K, int h, int &x2, int &y2)// K=0..7, (x2,y2):=   K   
{
	x2 = x0; y2 = y0;
	if (K == 0) { x2 += h; return; }
	if (K == 1) { x2 += h; y2 += h; return; }
	if (K == 2) { y2 += h; return; }
	if (K == 3) { x2 -= h; y2 += h; return; }
	if (K == 4) { x2 -= h; return; }
	if (K == 5) { x2 -= h; y2 -= h; return; }
	if (K == 6) { y2 -= h; return; }
	x2 += h; y2 -= h; return;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
void   copyN(int *src, int *dst, int n) {
	for (int i = 0; i < n; i++) *dst++ = *src++;
}
void   copyPnt(int *src, int *dst) { copyN(src, dst, 2); }
void   copyEdg(int *src, int *dst) { copyN(src, dst, 4); }
void   copyTri(int *src, int *dst) { copyN(src, dst, 6); }
void   copyBox(int *src, int *dst) { copyN(src, dst, 8); }

void   copyEdg(int *A, int *B, int *dst) { copyN(A, dst, 2); copyN(B, dst+2, 2); }
void   copyTri(int *A, int *B, int *C, int *dst) { copyN(A, dst, 2); copyN(B, dst+2, 2); copyN(C, dst+4, 2); }
void   copyBox(int *A, int *B, int *C, int *D, int *dst) { copyN(A, dst, 2); copyN(B, dst+2, 2); copyN(C, dst+4, 2); copyN(D, dst+6, 2); }

//-----------------------------------------------------------------------------
void   swapE(int *edge)					//   
{
	int tmp;										//   P2,P3
	tmp = edge[0]; edge[0] = edge[2]; edge[2] = tmp;
	tmp = edge[1]; edge[1] = edge[3]; edge[3] = tmp;
}
//-----------------------------------------------------------------------------
double lenE(int *edge)					//  
{
	double dx = edge[0] - edge[2];
	double dy = edge[1] - edge[3];
	return sqrt((double)(dx*dx + dy*dy));
}
double lenE(int x1, int y1, int x2, int y2)//  
{
	double dx = x2 - x1;
	double dy = y2 - y1;
	return sqrt((double)(dx*dx + dy*dy));
}
double lenE(int *P0, int *P1)					//  
{
	double dx = P1[0]-P0[0];
	double dy = P1[1]-P0[1];
	return sqrt((double)(dx*dx + dy*dy));
}
//-----------------------------------------------------------------------------
//   
//    / =-2
//   *
//    \ =2
double kAngle(int *edge)				//   
{
	int dx = edge[2] - edge[0];
	int dy = edge[3] - edge[1];
	if (dx == 0) {
		if (dy >= 0) return 1e99; else return -1e99;
	}
	return ((double)dy) / dx;
}
double CanglE(int *edge1, int *edge2)		//     ()
{
	return CanglE(edge1[2] - edge1[0], edge1[3] - edge1[1], edge2[2] - edge2[0], edge2[3] - edge2[1] );
}
double CanglE(int *A, int *B, int *C)			//   ABC
{
	return CanglE(A[0] - B[0], A[1] - B[1], C[0] - B[0], C[1] - B[1]);
}
double CanglE(int x0, int y0, int x1, int y1)	//    
{
	double L = x0*x0 + y0*y0; L *= x1*x1 + y1*y1;
	if (L == 0) return 0;
	double co = (x0*x1 + y0*y1) / sqrt(L);
	if (co > 1) co = 1;
	if (co < -1) co = -1;
	return co;
}
double CanglE(int *A, int *B, int *C, int *D)	//     [AB)  [CD)
{
	return CanglE(B[0] - A[0], B[1] - A[1], D[0] - C[0], D[1] - C[1]);
}
//-----------------------------------------------------------------------------
void   CSanglE(int x0, int y0, int x1, int y1, double &co, double &si)// co:=cos(angle), si:=sin(angle) 
{
	double L = x0*x0 + y0*y0; L *= x1*x1 + y1*y1;
	if (L == 0) { co = si = 0; return; }
	L = 1 / sqrt(L);
	co = (x0*x1 + y0*y1)*L;
	if (co > 1) co = 1;
	if (co < -1) co = -1;

	si = (x0*y1 - y0*x1)*L;
	if (si > 1) si = 1;
	if (si < -1) si = -1;
}
//-----------------------------------------------------------------------------
void   rotate(int *A, int *B, int *C, double angle)// C:=(B)  A   angle
{
	double ca = cos(angle);
	double sa = sin(angle);
	int dx = B[0] - A[0], dy = B[1] - A[1];
	C[0] = A[0] + round( ca*dx + sa*dy);
	C[1] = A[1] + round(-sa*dx + ca*dy);
}
void   rotate(int Ax, int Ay, int Bx, int By, int &Cx, int &Cy, double angle)// C:=(B)  A   angle
{
	double ca = cos(angle);
	double sa = sin(angle);
	int dx = Bx - Ax, dy = By - Ay;
	Cx = Ax + round( ca*dx + sa*dy);
	Cy = Ay + round(-sa*dx + ca*dy);
}

//-----------------------------------------------------------------------------
void   shiftE(int *A, int *B, int h, int *C, int *D)// [CD] :=  [AB]   h 
{
	int dx = B[0] - A[0], dy = B[1] - A[1];
	double L = dx*dx + dy*dy;
	if (L == 0) { copyPnt(A, C); copyPnt(A, D); return; }
	L = h / sqrt(L);
	int sx =-round(L*dy);
	int sy = round(L*dx);
	C[0] = A[0] + sx;
	C[1] = A[1] + sy;
	D[0] = B[0] + sx;
	D[1] = B[1] + sy;
}
void   shiftE(int x0, int y0, int x1, int y1, int h, int &x2, int &y2, int &x3, int &y3)// [23] :=  [01]   h 
{
	int dx = x1-x0, dy = y1-y0;
	double L = dx*dx + dy*dy;
	if (L == 0) { x2 = x3 = x0; y2 = y3 = y0; return; }
	L = h / sqrt(L);
	int sx = -round(L*dy);
	int sy = round(L*dx);
	x2 = x0 + sx;
	y2 = y0 + sy;
	x3 = x1 + sx;
	y3 = y1 + sy;
}
//-----------------------------------------------------------------------------
// x' = a0 + co*x + si*y
// y' = a1 - si*x + co*y
void   TransfE(int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3, double &a0, double &a1, double &co, double &si)//  ,  [01]->[23]
{
	int dx0 = x1 - x0, dy0 = y1 - y0;
	int iL = dx0*dx0 + dy0*dy0;
	if (iL == 0) { a0 = a1 = co = si = 0; return; }
	double L = 1. / iL;
	int dx2 = x3 - x2, dy2 = y3 - y2;
	co = (dy0*dy2 + dx2*dx0) *L;
	si = -(dx0*dy2 - dy0*dx2)*L;
	a0 = x2 - ( co*x0 + si*y0);
	a1 = y2 - (-si*x0 + co*y0);
}
//-----------------------------------------------------------------------------
// 1.   ,  a=b=c=0
// 2.  , .. a*a+b*b=1
void   lineEq(int *edge, double &a, double &b, double &c)//   ax+by+c   
{
	int dx = edge[2] - edge[0];
	int dy = edge[3] - edge[1];
	double den = dx*dx + dy*dy;
	if (den == 0) { a = b = c = 0; return; }			//  
	den = 1 / sqrt(den);
	a = (edge[1] - edge[3])*den;
	b = (edge[2] - edge[0])*den;
	c = (edge[0] * edge[3] - edge[1] * edge[2])*den;
}
void   lineEq(int *P0, int *P1, double &a, double &b, double &c)	//   ax+by+c   
{
	int dx = P1[0] - P0[0];
	int dy = P1[1] - P0[1];
	double den = dx*dx + dy*dy;
	if (den == 0) { a = b = c = 0; return; }			//  
	den = 1 / sqrt(den);

	a = (P0[1] - P1[1])*den;
	b = (P1[0] - P0[0])*den;
	c = (P0[0] * P1[1] - P0[1] * P1[0])*den;
}
//-----------------------------------------------------------------------------
double lineDist(int *edge, int *P)	//    P  ,  
{
	double x0 = edge[0] - P[0], y0 = edge[1] - P[1];
	double x1 = edge[2] - P[0], y1 = edge[3] - P[1];
	//     <x0,y0>-<x1,y>   (0,0)
	double dx = x1 - x0, dy = y1 - y0;
	double den = dx*dx + dy*dy;
	if (den == 0) return 0;
	return fabs(x0*y1 - y0*x1) / sqrt(den);
}
double lineDist(int *A, int *B, int *P)		//    P  ,   [A,B]
{
	double x0 = A[0] - P[0], y0 = A[1] - P[1];
	double x1 = B[0] - P[0], y1 = B[1] - P[1];
	//     <x0,y0>-<x1,y>   (0,0)
	double dx = x1 - x0, dy = y1 - y0;
	double den = dx*dx + dy*dy;
	if (den == 0) return 0;
	return fabs(x0*y1 - y0*x1) / sqrt(den);
}
//-----------------------------------------------------------------------------
// (x,y):=  ,   
// :
// 0 -  ,
// 1 -    ,
// 2 -      edge1
// 3 -      edge2
// 4 -      
int    intersection(int *edge1, int *edge2, double &x, double &y) // (x,y):=  ,   
{
	return intersection(edge1, edge1 + 2, edge2, edge2 + 1, x, y);
}
int    intersection(int *A, int *B, int *C, int *D, double &x, double &y)	// (x,y):=   (AB), (CD)
{
	int x1 = A[0], y1 = A[1], x2 = B[0], y2 = B[1];
	int x3 = C[0], y3 = C[1], x4 = D[0], y4 = D[1];
	double den = x2*y4 - x2*y3 - x1*y4 + x1*y3 - x4*y2 + x4*y1 + x3*y2 - x3*y1;
	if (den == 0) return 0;
	double a = (x4*y1 - x3*y1 - x1*y4 + x1*y3 + y4*x3 - y3*x4) / den;
	x = x1 + a*(x2 - x1);
	y = y1 + a*(y2 - y1);
	//bool b1 = pointInRect(x1, y1, x2, y2, x, y) == 1;
	//bool b2 = pointInRect(x3, y3, x4, y4, x, y) == 1;
	bool b1 = (x1<=x && x<=x2 && y1<=y && y <= y2);
	bool b2 = (x3<=x && x<=x4 && y3<=y && y <= y4);

	if (b1 && b2) return 4;
	if (b1) return 2;
	if (b2) return 3;
	return 1;
}
//-----------------------------------------------------------------------------
double distE0(int *edge, double x, double y)		//     
{
	double a, b, c;
	lineEq(edge, a, b, c);					//  : a*x + b*y + c = 0,  a^2 + b^2 = 1.
	//   :  b*x - a*y + c' = 0
	//     1- :  b*x - a*y + c1 = 0  c1 = a*edge[1] - b*edge[0]
	//     2- :  b*x - a*y + c2 = 0  c2 = a*edge[3] - b*edge[2]
	double c1 = a*edge[1] - b*edge[0];
	double c2 = a*edge[3] - b*edge[2];
	double t0 = b*x - a*y;
	int s1 = t0 + c1 > 0 ? 1 : -1;
	int s2 = t0 + c2 > 0 ? 1 : -1;
	if (s1*s2 < 0)		//  ,   (x,y)    
	{
		return fabs(a*x + b*y + c);
	}
	//    (x,y)   
	double dx = x - edge[0];
	double dy = y - edge[1];
	double d1 = dx*dx + dy*dy;
	dx = x - edge[2];
	dy = y - edge[3];
	double d2 = dx*dx + dy*dy;
	return d1 < d2 ? sqrt(d1) : sqrt(d2);
}
double distE(int *edge1, int *edge2)								//   . 0,  
{
	double x, y;
	int code = intersection(edge1, edge2, x, y);
	if (code == 4) return 0;
	double d  = distE0(edge1, edge2[0], edge2[1]);
	double d1;
	d1 = distE0(edge1, edge2[2], edge2[3]); if (d1 < d) d = d1;
	d1 = distE0(edge2, edge1[0], edge2[1]); if (d1 < d) d = d1;
	d1 = distE0(edge2, edge1[2], edge2[3]); if (d1 < d) d = d1;
	return d;
}

//-----------------------------------------------------------------------------
void   envelEdg(int &minX, int &minY, int &maxX, int &maxY, int *A, int *B) //  
{
	minX = maxX = A[0];
	setMinMax(B[0], minX, maxX);
	minY = maxY = A[1];
	setMinMax(B[1], minY, maxY);
}
void   envelEdg(int &minX, int &minY, int &maxX, int &maxY, int *edg)		//  
{
	envelEdg( minX, minY, maxX, maxY, edg, edg+2);
}
void   envelSqu(int &minX, int &minY, int &maxX, int &maxY, int *di)		//     
{
	int d2[4];
	secondDiag(di, d2);
	envelBox(minX, minY, maxX, maxY, di, di + 2, d2, d2 + 2);				//  4-
}
void   envelTri(int &minX, int &minY, int &maxX, int &maxY, int *A, int *B, int *C)//  
{
	minX = maxX = A[0];
	setMinMax(B[0], minX, maxX);
	setMinMax(C[0], minX, maxX);
	minY = maxY = A[1];
	setMinMax(B[1], minY, maxY);
	setMinMax(C[1], minY, maxY);
}
void   envelTri(int &minX, int &minY, int &maxX, int &maxY, int *tri)		//  
{
	envelTri( minX, minY, maxX, maxY, tri, tri+2, tri+4);
}
void   envelBox(int &minX, int &minY, int &maxX, int &maxY, int *A, int *B, int *C, int *D)//  4-
{
	minX = maxX = A[0];
	setMinMax(B[0], minX, maxX);
	setMinMax(C[0], minX, maxX);
	setMinMax(D[0], minX, maxX);
	minY = maxY = A[1];
	setMinMax(B[1], minY, maxY);
	setMinMax(C[1], minY, maxY);
	setMinMax(D[1], minY, maxY);
}
void   envelBox(int &minX, int &minY, int &maxX, int &maxY, int *box)				//  4-
{
	envelBox( minX, minY, maxX, maxY, box, box+2, box+4, box+6);
}

//-----------------------------------------------------------------------------
/*
 N  (x[i], y[i]). 

S(a,b,c) = SUM (a*x[i]+b*y[i]+c)^2;
            i


1   dS
-  ---- = SUM x[i]*(a*x[i]+b*y[i]+c);
2   da     i

1   dS
-  ---- = SUM y[i]*(a*x[i]+b*y[i]+c);
2   db     i

1   dS
-  ---- = SUM (a*x[i]+b*y[i]+c);
2   dc     i

 

s11 = SUM x[i]^2
s12 = SUM x[i]*y[i]
s22 = SUM y[i]^2
s10 = SUM x[i]
s20 = SUM y[i]
s30 = SUM 1 = N

 S(a,b,c)     

(s11  s12 s10)
(s12  s22 s20)
(s10  s20  N )

1   dS
-  ---- = a*s11 + b*s12 + c*s10;
2   da

1   dS
-  ---- = a*s12 + b*s22 + c*s20;
2   db

1   dS
-  ---- = a*s10 + b*s20 + c*N;
2   dc

    S   a^2 + b^2 =1.     c  ,      :

c =  -(a*s10+b*s20)/N;

      S ,  S(a,b)     

(r11  r12)
(r12  r22)



r11 = s11 - s10^2/N;
r12 = s12 - s10*s20/N;
r22 = s22 - s20^2/N;

    :

xi(x) = x^2 - f1*x + f0,                        (*)



f1 = r11+r22;
f2 = r11*r22 - r12^2;

  

D = f1^2 - 4*f2;

  S(a,b,c)  S(a,b)   , ,
 (*)     .    

       f1 - sqrt(D)
x1 = ---------------
           2

    (x1)     - 

(r11-x1  r12   )
(r12     r22-x1)

      (a,b):

(r11-x1)*a + r12    *b = 0
r12    *a +(r22-x1)*b = 0

*/
double fndLine(int N, const int *v, double &a, double &b, double &c)//  N>1    <v[0],v[1]>,...,<v[2*N-2],v[2*N-1]>.   a*x+b*y+c=0   .
{
	a = b = c = 0;
	if (N < 2) return 0;
	double s11 = 0, s12 = 0, s22 = 0, s10 = 0, s20 = 0;
	const double N1 = 1. / N;
	const int *vv = v;
	for (int i = 0; i < N; i++) {
		int x = *vv++, y = *vv++;
		s11 += x*x;
		s12 += x*y;
		s22 += y*y;
		s10 += x;
		s20 += y;
	}
	double r11 = s11 - s10*s10 * N1;
	double r12 = s12 - s10*s20 * N1;
	double r22 = s22 - s20*s20 * N1;

	double f1 = r11 + r22;
	double f2 = r11*r22 - r12*r12;
	double D = f1*f1 - 4 * f2;
	if (D < 0) D = 0;
	double x1 = 0.5*(f1 - sqrt(D));
	if (r12 != 0) {
		a = 1;
		b = -(r11 - x1) / r12;
	}
	else {
		if (r11 != 0) {
			a = 0;
			b = 1;
		}
		else {
			a = 1;
			b = 0;
		}
	}
	c = 1 / sqrt(a*a + b*b);	a *= c; b *= c;		// ,  a^2 + b^2 = 1
	c = -(a*s10 + b*s20) / N;
	double L = a*a*s11 + b*b*s22 + c*c*N + 2 * (a*b*s12 + a*c*s10 + b*c*s20);
	return L <= 0 ? 0 : sqrt(L*N1);
}// fndLine(int N, const int *v, double &a, double &b, double &c)//  N>1    <v[0],v[1]>,...,<v[2*N-2],v[2*N-1]>.   a*x+b*y+c=0   .
//-----------------------------------------------------------------------------
bool   SameP(int *p1, int *p2)				//    ?
{
	return (p1[0]==p2[0]) && (p1[1]==p2[1]);
}
bool   SameE(int *p1, int *p2)				//    ?
{
	if (SameP(p1, p2)) return SameP(p1 + 2, p2 + 2);
	if (SameP(p1, p2 + 2)) return SameP(p1 + 2, p2);
	return false;
}
bool   almostSameP(int *p1, int *p2, int h)		//     ?
{
	return (abs(p1[0] - p2[0]) + abs(p1[1] - p2[1])) <= h;
}
bool   almostSameE(int *p1, int *p2, int h)		//     ?
{
	if (almostSameP(p1, p2, h)) return almostSameP(p1 + 2, p2 + 2, h);
	if (almostSameP(p1, p2 + 2, h)) return almostSameP(p1 + 2, p2, h);
	return false;
}

bool   SameT(int *p1, int *p2)//    ?
{
	if (SameP(p1, p2)) {
		if (SameP(p1 + 2, p2 + 2) && SameP(p1 + 4, p2 + 4)) return true;
		if (SameP(p1 + 4, p2 + 2) && SameP(p1 + 2, p2 + 4)) return true;
		return false;
	}

	if (SameP(p1, p2+2)) {
		if (SameP(p1 + 2, p2) && SameP(p1 + 4, p2 + 4)) return true;
		if (SameP(p1 + 4, p2) && SameP(p1 + 2, p2 + 4)) return true;
		return false;
	}

	if (SameP(p1, p2 + 4)) {
		if (SameP(p1    , p2) && SameP(p1 + 2, p2 + 2)) return true;
		if (SameP(p1 + 2, p2) && SameP(p1    , p2 + 2)) return true;
		return false;
	}
	return false;
}
/*
//-----------------------------------------------------------------------------
edgeIterator::edgeIterator(int x0_, int y0_, int x1_, int y1_)
{
	x0 = x0_; y0 = y0_; x1 = x1_; y1 = y1_;
	deltaX = abs(x1 - x0);
	deltaY = abs(y1 - y0);
	signX = x0 < x1 ? 1 : -1;
	signY = y0 < y1 ? 1 : -1;
	error = deltaX - deltaY;
}
edgeIterator::edgeIterator(int *P0, int *P1)
{
	x0 = P0[0]; y0 = P0[1]; x1 = P1[0]; y1 = P1[1];
	deltaX = abs(x1 - x0);
	deltaY = abs(y1 - y0);
	signX = x0 < x1 ? 1 : -1;
	signY = y0 < y1 ? 1 : -1;
	error = deltaX - deltaY;
}
bool edgeIterator::next()		// ,   
{
	if (x0 == x1 && y0 == y1) return true;
	const int error2 = error * 2;
	if (error2 > -deltaY) {
		error -= deltaY;
		x0 += signX;
	}
	if (error2 < deltaX) {
		error += deltaX;
		y0 += signY;
	}
	return false;
}
*/
//-----------------------------------------------------------------------------
int vectProd(int x1, int y1, int x2, int y2) {
	return x1*y2 - x2*y1;
}
//-----------------------------------------------------------------------------
bool   isIn(int x0, int y0, int x1, int y1, int x, int y)//    (x,y)   <(x0,y0)-(x1,y1)>
{
	return  x0 <= x && x <= x1 && y0 <= y && y <= y1;
}
//-----------------------------------------------------------------------------
bool   edgFar(int *A, int *B, int *C, int *D, int h)//  [AB]  [CD]  (h)   ?  
{
	int minX, minY, maxX, maxY;
	minX = maxX = A[0];
	minY = maxY = A[1];
	setMinMax(B[0], minX, maxX);
	setMinMax(B[1], minY, maxY);
	minX -= h; maxX += h;
	minY -= h; maxY += h;
	if (C[0] < minX && D[0] < minX) return true;
	if (C[0] > maxX && D[0] > maxX) return true;
	if (C[1] < minY && D[1] < minY) return true;
	if (C[1] > maxY && D[1] > maxY) return true;
	return false;
}
bool   edgFar(int *e1, int *e2, int h)			//  e1  e2  (h)   ?  
{
	return edgFar(e1, e1 + 2, e2, e2 + 2, h);
}
//-----------------------------------------------------------------------------
double joinE(int *A, int *B, int *C, int *D, int *join)	//  4-  ,  .   
{
	double r, r1;
	copyPnt(A, join); copyPnt(B, join + 2); r = lenE(join);			// [A,B]
	r1 = lenE(A, C); if (r1 > r) { r = r1; copyPnt(C, join + 2); }	// [A,C]
	r1 = lenE(A, D); if (r1 > r) { r = r1; copyPnt(D, join + 2); }	// [A,D]
	r1 = lenE(B, C); if (r1 > r) { r = r1; copyPnt(B, join);  copyPnt(C, join + 2); }	// [B,C]
	r1 = lenE(B, D); if (r1 > r) { r = r1; copyPnt(B, join);  copyPnt(D, join + 2); }	// [B,D]
	r1 = lenE(C, D); if (r1 > r) { r = r1; copyPnt(C, join);  copyPnt(D, join + 2); }	// [C,D]

	double u, v, w;
	lineEq(join, u, v, w);
	r  = fabs(u*A[0] + v*A[1] + w);
	r1 = fabs(u*B[0] + v*B[1] + w); if (r1 > r) r = r1;
	r1 = fabs(u*C[0] + v*C[1] + w); if (r1 > r) r = r1;
	r1 = fabs(u*D[0] + v*D[1] + w); if (r1 > r) r = r1;
	return r;
}
double joinE(int *e1, int *e2, int *join)		// join:=(e1,e2),  .  e1,e2   
{
	return joinE(e1, e1 + 2, e2, e2 + 2, join);
}
//-----------------------------------------------------------------------------
/*
bool   reduceTo(int *edge, int x0, int y0, int x1, int y1)//  edge   <(x0,y0)-(x1,y1)>
{
	bool isIn0 = isIn(x0, y0, x1, y1, edge[0], edge[1]);		//      
	bool isIn1 = isIn(x0, y0, x1, y1, edge[2], edge[3]);		//      
	if (!isIn0) {
		edgeIterator ei(edge[0], edge[1], edge[2], edge[3]);
		do {
			if (isIn(x0, y0, x1, y1, ei.X(), ei.Y())) break;
		} while (!ei.next());
		edge[0] = ei.X();
		edge[1] = ei.Y();
	}
	if (!isIn1) {
		edgeIterator ei(edge[2], edge[3], edge[0], edge[1]);
		do {
			if (isIn(x0, y0, x1, y1, ei.X(), ei.Y())) break;
		} while (!ei.next());
		edge[2] = ei.X();
		edge[3] = ei.Y();
	}
	if (edge[0] == edge[2] && edge[1] == edge[3] && !isIn(x0, y0, x1, y1, edge[0], edge[1])) return false;
	return true;
}
*/
//-----------------------------------------------------------------------------
void   expandEdg(int *A, int *B, int h)		//   [A,B]   B  h 
{
	int dx = B[0]-A[0], dy = B[1]-A[1];
	double s = h / sqrt((double)(dx*dx + dy*dy));
	B[0] += round(s*dx);
	B[1] += round(s*dy);
}
void   expandEdg(int x0, int y0, int &x1, int &y1, double q)//   [01]   1  q 
{
	x1 = x0 + round(q*(x1-x0));
	y1 = y0 + round(q*(y1-y0));
}
//-----------------------------------------------------------------------------
double edgProj( int *A, int *B, int *P)		// x: (P)   (AB)  x*A +(1-x)*B 
{
	int x0 = B[0]-A[0], y0=B[1]-A[1];
	int x1 = P[0]-A[0], y1=P[1]-A[1];
	return (x0*x1+y0*y1)/((double)(x0*x0+y0*y0));
}
//-----------------------------------------------------------------------------
bool edgAdd1( int *e1, int *e2, int h1) //    e2  e1
{
	double p;
	p = edgProj(e1, e1+2, e2);
	if( p<0 || p>1) return false;
	p = edgProj(e1, e1+2, e2+2);
	if( p<0 || p>1) return false;
	//   e2    e1
	if( lineDist(e1, e2) > h1 ) return false;
	if( lineDist(e1, e2+2) > h1 ) return false;
	return true;
}

// e1:=         .
//          h1   
//         h2
// e1 = [AB], e2=[CD]
bool   edgAdd( int *e1, int *e2, int h1, int h2)//    e1   e2 (      h1, h2) 
{
	double t;
	if(edgAdd1(e1, e2, h1)) return true;	// e2   e1 
	if(edgAdd1(e2, e1, h1)) {				// e1   e2 
		copyEdg(e2, e1);					// e1:=e2;
		return true;
	}
	//        
	double len1 = lenE(e1);
	double len2 = lenE(e2);
	int r1[4], r2[4];
	if( len1>=len2 ) { copyEdg(e1, r1); copyEdg(e2, r2); }
	else             { copyEdg(e2, r1); copyEdg(e1, r2); }
	//  r1 -   ,  r2 -  
	double x2 = edgProj(r1, r1+2, r2);
	double x3 = edgProj(r1, r1+2, r2+2);
	//   (r1) : 0, 1, x2, x3
	if( x2>x3 ) {
		swapE(r2);
		t=x2; x2=x3; x2=t;
	}
	//  x2<=x3
	if( x3>1) //  r2   r1
	{
		//   [r1,r2+2]
		if( lineDist(r1, r2+2, r1+2)>h1 ) return false;
		if( lineDist(r1, r2+2, r2)>h1 ) return false;
		if( x2>1 && distE(r1+2, r2)>h2 ) return false;
		//  .  
		copyPnt(r1, e1);
		copyPnt(r2+2, e1+2);
		return true;
	}
	//  r2   r1
	//   [r2,r1+2],   : r2+2, r1
	if( lineDist(r2, r1+2, r1)>h1 ) return false;
	if( lineDist(r2, r1+2, r2+2)>h1 ) return false;
	if( x3<0 && distE(r2+2, r1)>h2 ) return false;
	//  .  
	copyPnt(r2, e1);
	copyPnt(r1+2, e1+2);
	return true;
} // edgAdd( int *e1, int *e2, int h1, int h2)//    e1   e2 (      h1, h2) 
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
double areaT(int*A, int *B, int *C) //   - ABC
{
	int x1 = B[0] - A[0], y1 = B[1] - A[1];
	int x2 = C[0] - A[0], y2 = C[1] - A[1];
	return 0.5*(x1*y2 - x2*y1);
}
int    areaT2(int*A, int *B, int *C)	// 2*  - ABC
{
	int x1 = B[0] - A[0], y1 = B[1] - A[1];
	int x2 = C[0] - A[0], y2 = B[1] - A[1];
	return x1*y2 - x2*y1;
}
//-----------------------------------------------------------------------------
bool   inAngle(int *A, int *B, int *C, int *P)		//  P   ABC? (  )
{
	int xa = A[0] - B[0], ya = A[1] - B[1];
	int xc = C[0] - B[0], yc = C[1] - B[1];
	int x = P[0] - B[0], y = P[1] - B[1];
	//    (x,y)   (xa,ya)-(0,0)-(xc,yc)?
	int sAC = vectProd(xa, ya, xc, yc);
	int sAP = vectProd(xa, ya, x, y);
	int sPC = vectProd(x, y, xc, yc);
	// sAP, sPC     c sAC
	return sAP*sAC >= 0 && sPC*sAC >= 0;
}
//-----------------------------------------------------------------------------
bool   inTriangle(int x1, int y1, int x2, int y2, int x3, int y3) //  (0,0)  ?
{
	int d1 = vectProd(x1, y1, x2, y2);
	int d2 = vectProd(x2, y2, x3, y3);
	int d3 = vectProd(x3, y3, x1, y1);
	if (d1 < 0) { d2 *= -1; d3 *= -1; }
	return d2 >= 0 && d3 >= 0;
}
bool   inTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int x, int y) // (x,y)  ?
{
	return inTriangle(x1 - x, y1 - y, x2 - x, y2 - y, x3 - x, y3 - y);
}
bool   inTriangle(int *P1, int *P2, int *P3, int *P)		//  (P,P+1)   (P1,P1+1),(P2,P2+1),(P3,P3+1)?
{
	return inTriangle(P1[0], P1[1], P2[0], P2[1], P3[0], P3[1], P[0], P[1]);
}
//////////////////////////////////////////////////////////////////////////////////
// (a1,a2,a3) -    v   <v1,v2,v3>
void baricentr( int *v, int *v1, int *v2, int *v3,
				double &a1, double &a2,  double &a3 )
{
	double den = -v1[0]*v3[1] + v1[0]*v2[1] - v2[0]*v1[1] + v2[0]*v3[1] + v3[0]*v1[1] - v3[0]*v2[1];
	if( den==0 ) {a1=a2=a3=-1; return; } //  
	den = 1/den;
	a1  = (v[1] *v3[0] - v3[1]*v[0]  - v2[0]*v[1]  + v[0] *v2[1] + v2[0]*v3[1] - v3[0]*v2[1])*den;
	a2  = (v3[0]*v1[1] - v[0] *v1[1] + v1[0]*v[1]  - v1[0]*v3[1] - v[1] *v3[0] + v3[1]*v[0] )*den;
	a3  = (v1[0]*v2[1] - v1[0]*v[1]  - v2[0]*v1[1] + v2[0]*v[1]  + v[0] *v1[1] - v[0] *v2[1])*den;
}

//-----------------------------------------------------------------------------
// 4-  (array of 8 integer)
//-----------------------------------------------------------------------------
bool    makeBox(int *A, int *B, int *C, int *D, int *box)	//  4-    4-. 
{
	double fx, fy, f0;				// fx*x + fy*y + f0 = 0
	double v1, v2;
	lineEq(A, C, fx, fy, f0);			
	v1 = fx*B[0] + fy*B[1] + f0;
	v2 = fx*D[0] + fy*D[1] + f0;
	if (v1*v2 < 0) {				// AC - 
		copyBox(A, B, C, D, box);
		return convex(box);
	}

	lineEq(A, B, fx, fy, f0);
	v1 = fx*C[0] + fy*C[1] + f0;
	v2 = fx*D[0] + fy*D[1] + f0;
	if (v1*v2 < 0) {				// AB - 
		copyBox(A, C, B, D, box);
		return convex(box);
	}

	lineEq(A, D, fx, fy, f0);
	v1 = fx*B[0] + fy*B[1] + f0;
	v2 = fx*C[0] + fy*C[1] + f0;
	if (v1*v2 < 0) {				// AD - 
		copyBox(A, B, D, C, box);
		return convex(box);
	}

	return false;
}
//-----------------------------------------------------------------------------
double CangleB(int *box, int k)		//    k-   
{
	int edge30[4] = { box[6],box[7], box[0], box[1] };
	if (k == 0) {
		return -CanglE(edge30, box);
	}
	else if (k == 1)
		return -CanglE(box + 2, box);
	else if (k == 2)
		return -CanglE(box + 4, box + 2);
	else if (k == 3)
		return -CanglE(box + 4, edge30);
	else return 1e99;
}
//-----------------------------------------------------------------------------
double lenB(int *box, int k)			//   k-   k+1
{
	int edge30[4] = { box[6],box[7], box[0], box[1] };
	if (k == 0) return lenE(box);
	else if (k == 1)	return lenE(box + 2);
	else if (k == 2)	return lenE(box + 4);
	else if (k == 3)	return lenE(edge30);
	else return -1e99;
}
//-----------------------------------------------------------------------------
double areaB(int *box)					// 
{
	return fabs(areaT(box, box + 2, box + 4))
		+ fabs(areaT(box, box + 4, box + 6));
}
double areaB(int *A, int *B, int *C, int *D) //  ABCD
{
	return fabs(areaT(A,B,C))+ fabs(areaT(A,C,D));
}
//-----------------------------------------------------------------------------
bool   convex(int *A, int *B, int *C, int *D)// ?
{
	if (!inAngle(D, A, B, C)) return false;
	if (!inAngle(A, B, C, D)) return false;
	if (!inAngle(B, C, D, A)) return false;
	if (!inAngle(C, D, A, B)) return false;
	return true;
}
bool   convex(int *box)				// ?
{
	return convex(box, box + 2, box + 4, box + 6);
}
//-----------------------------------------------------------------------------
void   shiftB(int *box)				//    
{
	int b0 = box[0], b1 = box[1];
	for (int i = 0; i < 6; i++)	box[i] = box[i + 2];
	box[6] = b0; box[7] = b1;
}
//-----------------------------------------------------------------------------
void   boxLengthM(int *box, double &min, double &max)//      
{
	double r;
	min = 1e99; max = -1e99;
	for (int i = 0; i < 4; i++) {
		r = lenB(box, i);
		if (r < min) min = r;
		if (r > max) max = r;
	}
}
void   boxCAngleM(int *box, double &min, double &max)//      
{
	double r;
	min = 1e99; max = -1e99;
	for (int i = 0; i < 4; i++) {
		r = CangleB(box, i);
		if (r < min) min = r;
		if (r > max) max = r;
	}
}
//-----------------------------------------------------------------------------
void boxCenter(int *box, int &x, int &y)			// (x,y):=(box)
{
	x = (box[0] + box[2] + box[4] + box[6] + 1) / 4;
	y = (box[1] + box[3] + box[5] + box[7] + 1) / 4;
}
//-----------------------------------------------------------------------------
void shrink(int *src, int *dst, int h)			// dst:= box(src)  h   
{
	int c[2];
	boxCenter(src, c[0], c[1]);
	copyBox(src, dst);
	for (int k = 0; k < 8; k += 2) {
		expandEdg(c,  dst+k, -h);
	}
}
//-----------------------------------------------------------------------------
bool   inBox(int *box, int *P)			//  P   ?
{
	if (inTriangle(box, box + 2, box + 4, P)) return true;
	if (inTriangle(box, box + 4, box + 6, P)) return true;
	return false;
}
//-----------------------------------------------------------------------------
bool  InBox(int *box, int x, int y)		//    (x,y)  box
{
	return
		inTriangle(box[0], box[1], box[2], box[3], box[4], box[5], x, y) ||
		inTriangle(box[0], box[1], box[4], box[5], box[6], box[7], x, y);
}
//-----------------------------------------------------------------------------
void   secondDiag(int *d1, int *d2)		//    d1,    d2
{
	int dx = d1[2] - d1[0], dy = d1[3] - d1[1];
	d2[0] = d1[0] + (dx-dy)/2; d2[1] = d1[1] + (dx+dy)/2;
	d2[2] = d1[0] + (dx+dy)/2; d2[3] = d1[1] + (-dx+dy)/2;
}
//-----------------------------------------------------------------------------
void   getDiag(int *s, int *d)			//     s,   d
{
	int dx = (s[2] - s[0])/2, dy =(s[3] - s[1])/2;
	d[0] = s[0] - dy; d[1] = s[1] + dx;
	d[2] = s[2] + dy; d[3] = s[3] - dx;
}
void   getDiag(int xa, int ya, int xc, int yc, int &x0, int &y0, int &x1, int &y1)
{
	int dx = (xc - xa) / 2, dy = (yc - ya) / 2;
	x0 = xa - dy; y0 = ya + dx;
	x1 = xc + dy; y1 = yc - dx;
}
//-----------------------------------------------------------------------------
bool   almostSameSq(int *d1, int *d2)		//    .    ?
{
	double L1 = sqrt(0.5)*hy2(d1[2] - d1[0], d1[3] - d1[1]);	//  
	double L2 = sqrt(0.5)*hy2(d2[2] - d2[0], d2[3] - d2[1]);	//  
	int cx1 = d1[0] + d1[2], cy1 = d1[1] + d1[3];				//   
	int cx2 = d2[0] + d2[2], cy2 = d2[1] + d2[3];				//   
	double cL = 0.5*hy2(cx2 - cx1, cy2 - cy1);					//   

	
	if (cL < 0.2*L1 && cL < 0.2*L2) return true;		//    

	if (L1 < L2) {										// L1<L2
		if (cL < 0.9*L2) return true;					// 
	}
	else {
		if (cL < 0.9*L1) return true;

	}
	return false;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
