// readCSV.cpp   : 2018-11-04 read *.csv file to vector<vector<string>>
// (c) Khashin S.I.  khash2@gmail.com

#pragma warning(disable : 4996)

#include <stdio.h>
#include <vector>
#include <string>

using namespace std;
#include "readCSV.h"				// read *.csv file to vector<vector<string>>

//-----------------------------------------------------------------------------
int ML_readCSV(const char *fname, vector<vector<string>> &data, char delim)	// read *.csv file to vector<vector<string>>
{
	data.clear();
	vector<vector<string>> dat0;
	FILE *f = fopen(fname, "rb");
	if (!f) return -1;
	fseek(f, 0, SEEK_END);
	int fsize = ftell(f);				// size of the file. Must be  <2G!
	fseek(f, 0, SEEK_SET);
	char *buf = new char[fsize+1];
	fread(buf, 1, fsize, f);			// read file to buffer
	buf[fsize] = 0;						// null-terminated string
	fclose(f);
	vector<string> line;				// current record
	string item;						// current item
	char *p = buf;						// 
	const char Q1 = 39;					// quote '
	const char Q2 = 34;					// quote  "
	const char Q3 = 96;					// quote  `
	char ch;							// current char
	int quote = 0;						// >0 - read item in quote
	while (*p) {
		ch = *p++;
		if (quote>0) {					// read item in quote
			if( ch == quote) {			// current char is quote
				if (*p == quote) {		// if next char also quote
					item.push_back(ch);
					p++;				// skip next char
					continue;
				}
				// now next char is not quote, that is it is the end of quote
				quote = 0;
				continue;
			}
			// current char in not quote
			item.push_back(ch);
			continue;
		}
		// now quote==0
		if (ch == '\n' || ch == '\r') {	// end of line
			if (item.size() > 0) {
				line.push_back(item);
				item.clear();
			}
			if (line.size() > 0) {
				dat0.push_back(line);
				line.clear();
			}
			continue;
		}
		//  ch -   
		if (ch == delim) {				// delimeter
			line.push_back(item);
			item.clear();
			while (*p == ' ') p++;
			continue;
		}
		// now ch not eol and not delim
		if (item.size() == 0 && (ch == ' ' || ch == '\t'))
			continue;					// skip intial blanks
		if (ch == Q1) {
			quote = Q1;
			continue;
		}
		if (ch == Q2) {
			quote = Q2;
			continue;
		}
		if (ch == Q3) {
			quote = Q3;
			continue;
		}
		// now usual char:
		item.push_back(ch);
	}
	delete[]buf;
	if( item.size()>0) line.push_back(item);
	if (line.size()>0) dat0.push_back(line);
	int my = (int)dat0.size();
	int mx = 0;					// number of columns
	for (int y = 0; y < (int)dat0.size(); y++) {
		int mx0 = (int)dat0[y].size();
		if (mx0 > mx) mx = mx0;
		for (int x = 0; x < mx0; x++) {
			dat0[y][x].erase(dat0[y][x].find_last_not_of(" \t") + 1);
		}
	}
	// add empty elements, if need
	item.clear();
	for (int y = 0; y < (int)dat0.size(); y++) {
		int mx0 = (int)dat0[y].size();
		if (mx0 == mx) continue;
		for (int x = mx0; x < mx; x++) {
			dat0[y].push_back(item);
		}
	}
	// fill data
	for (int x = 0; x < mx; x++) {
		line.clear();
		for (int y = 0; y < (int)dat0.size(); y++) 
			line.push_back(dat0[y][x]);
		data.push_back(line);
	}
	return my;
}
//-----------------------------------------------------------------------------
int ML_readCSV(const char *fname, vector<string> &header, vector<vector<string>> &data, char delim)	// read *.csv file to vector<vector<string>>
{
	int my = ML_readCSV(fname, data, delim);
	int mx = (int)data.size();
	header.clear();
	for (int x = 0; x < mx; x++) {
		header.push_back(data[x][0]);
		data[x].erase(data[x].begin());
	}
	return my-1;
}
//