// vTransf.cpp: ,    
// 2006-11-08
//
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "vUtil.h"			//   vError
#include "vTransf.h"		//  

////////////////////////////////////////////////////////////////
CvTransf::CvTransf()		// ,   
{
	//this->set(0,0);
	this->setNothing();
}

CvTransf::CvTransf(double dx, double dy)	// ,  
{
	this->set(dx,dy);
}

CvTransf::CvTransf(double dx, double dy, double angle)	// ,  
{
	this->set(dx,dy,angle);
}

CvTransf::CvTransf(double a0, double a1, double a2, double a3, double a4, double a5)	// ,  
{
	this->set(a0,a1,a2,a3,a4,a5);
}

CvTransf::~CvTransf()	// 
{
	this->kind = tTransf_none;
}

///////////////////////////////////////////////////////////////////
void CvTransf::set(double dx, double dy)					//  
{
	this->kind = tTransf_shift;
	a[0] = dx;
	a[1] = dy;
	a[2] = a[3] = a[4] = a[5] = 0;
}
void CvTransf::set(double dx, double dy, double angle)	//  
{
	this->kind = tTransf_rotate;
	a[0] = dx;
	a[1] = dy;
	a[2] = angle;
	a[3] = a[4] = a[5] = 0;
}
void CvTransf::set(double a0, double a1, double a2, double a3, double a4, double a5)	//  
{
	this->kind = tTransf_affine;
	a[0] = a0;
	a[1] = a1;
	a[2] = a2;
	a[3] = a3;
	a[4] = a4;
	a[5] = a5;
}

void CvTransf::setRot(double x0, double y0, double angle)//     (x0,y0)
{
	this->kind = tTransf_rotate;
	double co = cos(angle);
	double si = sin(angle);

	a[0] = x0*(1-co) - y0*si;
	a[1] = x0*si + y0*(1-co);
	a[2] = angle;
	a[3] = a[4] = a[5] = 0;
}
//  + ,  P0->P2, P1->P3
void CvTransf::setSRot(double x0, double y0, double x1, double y1, double x2, double y2, double x3, double y3)//  + ,  P0->P2, P1->P3
{
	this->kind = tTransf_affine;
	double dx0 = x1 - x0, dy0 = y1 - y0;
	double iL = dx0*dx0 + dy0*dy0;
	if (iL == 0) { set(0,0); return; }
	double L = 1. / iL;
	double dx2 = x3 - x2, dy2 = y3 - y2;
	double co = (dy0*dy2 + dx2*dx0) *L;
	double si = -(dx0*dy2 - dy0*dx2)*L;
	a[0] = x2 - (co*x0 + si*y0);
	a[1] = co;
	a[2] = si;
	a[3] = y2 - (-si*x0 + co*y0);
	a[4] =-si;
	a[5] = co;
}

void CvTransf::setAff(double x0, double y0, double a1, double a2, double a4, double a5)//     (x0,y0)
{
	this->kind = tTransf_affine;
	a[0] = x0*(1-a1) - y0*a2;
	a[3] =-x0*a4     + y0*(1-a5);
	a[1] = a1;
	a[2] = a2;
	a[4] = a4;
	a[5] = a5;
}

void CvTransf::addShift(double vx, double vy)			//  
{
	if     ( kind == tTransf_shift  ) {a[0] += vx; a[1] += vy;}
	else if( kind == tTransf_rotate ) {a[0] += vx; a[1] += vy;}
	else							  {a[0] += vx; a[3] += vy;}
}

double CvTransf::get(int i)								//   i
{
	if( i<0 || i>=N() ) vError("CvTransf::get: i=%d", i);
	return this->a[i];
}

void CvTransf::inc(int i, double v)
{
	if (i<0 || i >= N()) vError("CvTransf::get: i=%d", i);
	this->a[i] += v;
}


//   ,  
//	A(x1, y1) = (x1s, y1s), 
//	A(x2, y2) = (x2s, y2s), 
//	A(x3, y3) = (x3s, y3s), 
void CvTransf::setAffine(double x1,  double y1,   double x2,  double y2,   double x3,  double y3,  //   ,  
			 			 double x1s, double y1s,  double x2s, double y2s,  double x3s, double y3s)
{
	this->kind = tTransf_affine;
	affineTMove( x1,y1, x2,y2, x3,y3, x1s,y1s, x2s,y2s, x3s,y3s, 
		a[0], a[1], a[2], a[3], a[4], a[5] );
}


///////////////////////////////////////////////////////////////////
int  CvTransf::N()	//  
{
	if     ( kind == tTransf_none   ) return 0;
	else if( kind == tTransf_shift  ) return 2;
	else if( kind == tTransf_rotate ) return 3;
	else							  return 6;

}

///////////////////////////////////////////////////////////////////
void CvTransf::setArr(double *b, int n_)			//   
{
	if     ( n_==2) kind = tTransf_shift;
	else if( n_==3) kind = tTransf_rotate;
	else if( n_==6) kind = tTransf_affine;
	else vError("CvTransf::setArr: n_=%d", n_);
	for( int i=0; i<n_; i++ ) this->a[i] = b[i];
}

///////////////////////////////////////////////////////////////////
// this->a[0..n-1] += v*b[0..n-1]
void CvTransf::addArr(double *b, int n_, double v)	//    
{
	if( n_ != this->N() ) vError("CvTransf::setArr: n=%d, n_=%d", this->N(), n_);
	for( int i=0; i<n_; i++ ) this->a[i] += v*b[i];
}

///////////////////////////////////////////////////////////////////
void CvTransf::set1()	//   
{
	this->set(0,0);
}

///////////////////////////////////////////////////////////////////
void CvTransf::copyFrom( CvTransf &b)
{
	//this->set1();
	this->kind = b.kind;
	for( int i=0; i<this->N(); i++ ) this->a[i] = b.a[i];
}

///////////////////////////////////////////////////////////////////
// .     :
//  rotate.kind = tTransf_rotate.    :
// rotate.convertFrom( rotate, tTransf_affine); 
void CvTransf::convertFrom( CvTransf &b, tTransf knd) // this= b   knd.  knd  ,    this->kind;
{
	if( knd==tTransf_none ) { this->kind=knd; return; }
	//  knd=this->kind - ,     
	if( knd == tTransf_shift )
	{
		if( b.kind == tTransf_shift ) { a[0] = b.a[0]; a[1] = b.a[1]; }
		else vError("CvTransf::convertFrom: kind=%d, b.kind=%d", kind, b.kind);
	} 
	else if( knd == tTransf_rotate )
	{
		if     ( b.kind == tTransf_shift  ) { a[0] = b.a[0]; a[1] = b.a[1]; a[2] = 0; }
		else if( b.kind == tTransf_rotate ) { a[0] = b.a[0]; a[1] = b.a[1]; a[2]=b.a[2]; }
		else vError("CvTransf::convertFrom: kind=%d, b.kind=%d", kind, b.kind);
	} 
	else //if( knd == tTransf_affine )
	{
		if     ( b.kind == tTransf_shift  ) { a[0]=b.a[0]; a[3]=b.a[1]; a[1]=1; a[2]=0; a[4]=0; a[5]=1; }
		else if( b.kind == tTransf_rotate ) 
		{ 
			double co = cos(b.a[2]);
			double si = sin(b.a[2]);
			a[0]=b.a[0]; a[3]=b.a[1]; 
			a[1]= co; a[2]=si; 
			a[4]=-si; a[5]=co; 
		}
		else copyFrom(b);
    }
	this->kind=knd;
}
///////////////////////////////////////////////////////////////////
// (x0,y0) :=  (apply)    (x,y) 
void CvTransf::apply( double x, double y,  double &x0, double &y0 )
{
	if( kind == tTransf_shift )
	{
		x0 = x + this->a[0];
		y0 = y + this->a[1];
	} 
	else if( kind == tTransf_rotate )
	{
		double co = cos(a[2]);
		double si = sin(a[2]);
		x0 = a[0] + co*x + si*y;
		y0 = a[1] - si*x + co*y;
	} 
	else //if( kind == tTransf_affine )
	{
		x0 = a[0] + a[1]*x + a[2]*y;
		y0 = a[3] + a[4]*x + a[5]*y;
    }
}
////////////////////////////////////////////////////////
void CvTransf::save( FILE *f )									//  CvTransf   
{
	if( kind == tTransf_shift )
	{
		fprintf( f, "    --------------CvShift: (vx, vy)=(%12.6f,%12.6f) \n", a[0], a[1]);
	} 
	else if( kind == tTransf_rotate )
	{
		double co = cos(a[2]);
		double si = sin(a[2]);
		fprintf( f, "    --------------CvRotate      -------\n");
		fprintf( f, "    x' =%12.6f + %12.6f*x + %12.6f*y\n", a[0],  co, si);
		fprintf( f, "    y' =%12.6f + %12.6f*x + %12.6f*y\n", a[1], -si, co);
		fprintf( f, "    angle = %12.6f ---------------------\n", a[2]);
	} 
	else //if( kind == tTransf_affine )
	{
		fprintf( f, "    ------CvAffine------\n");
		fprintf( f, "    x' =%12.8f + %12.8f*x + %12.8f*y\n", a[0], a[1], a[2]);
		fprintf( f, "    y' =%12.8f + %12.8f*x + %12.8f*y\n", a[3], a[4], a[5]);
		fprintf( f, "    ---------------------\n");
    }
}
////////////////////////////////////////////////////////
void CvTransf::save( const char *fname ) // append CvTransf to file
{
	FILE *f=fopen(fname, "a");
	if( !f ) vError("CvTransf::save: file %s not open", fname);
	this->save(f);
	fclose(f);
}
//     (x,y)  transfX, transfY   #k
double CvTransf::diffX( double x, double y, int k )
{
	if( kind == tTransf_shift )
	{
		return (k==0 ? 1 : 0);
	} 
	else if( kind == tTransf_rotate )
	{
		return k==0? 1 : k==1? 0 : -sin(a[2])*x + cos(a[2])*y;
	} 
	else //if( kind == tTransf_affine )
	{
		return k==0? 1 : k==1? x : k==2? y : 0 ;
    }
}
double CvTransf::diffY( double x, double y, int k )
{
	if( kind == tTransf_shift )
	{
		return (k==1 ? 1 : 0);
	} 
	else if( kind == tTransf_rotate )
	{
		return k==0? 0 : k==1? 1 : -cos(a[2])*x - sin(a[2])*y;
	} 
	else //if( kind == tTransf_affine )
	{
		return k==3? 1 : k==4? x : k==5? y : 0 ;
    }
}
///////////////////////////////////////////////////////////////////////////////
void CvTransf::mult( double k )		//    k 
{
	if( this->kind != tTransf_affine )
	{
		a[0] *= k;
		a[1] *= k;
	} else
	{
		a[0] *= k;
		a[3] *= k;
	}
}
////////////////////////////////////////////////////////////////
void CvTransf::mult( CvTransf &a_, CvTransf &b_)	// this = a*b.  ,  
{
	CvTransf a, b;			//    ,   
	a.copyFrom(a_);
	b.copyFrom(b_);
	if( a.kind < b.kind ) a.convertFrom( a, b.kind );
	if( a.kind > b.kind ) b.convertFrom( b, a.kind );
	//  a  b   
	//CvTransf c;
	//c.convertFrom( b, this->kind );
	if( a.kind == tTransf_shift )
	{
		this->set(a.a[0]+b.a[0], a.a[1] + b.a[1] );
	} 
	else if( a.kind == tTransf_rotate )
	{
		this->kind = tTransf_rotate;

		double ang= a.a[2];
		double co = cos(ang);
		double si = sin(ang);

		double a0 = a.a[0] + co*b.a[0] + si*b.a[1];
		double a1 = a.a[1] - si*b.a[0] + co*b.a[1];

		this->a[0] = a0;
		this->a[1] = a1;
		this->a[2] = a.a[2] + b.a[2];
	} 
	else if( a.kind == tTransf_affine )
	{
		this->kind = tTransf_affine;
		double *aa = a.a;
		double *bb = b.a;

		double a0 = aa[0]+aa[1]*bb[0]+aa[2]*bb[3];
		double a1 = aa[1]*bb[1]+aa[2]*bb[4];
		double a2 = aa[1]*bb[2]+aa[2]*bb[5];

		double a3 = aa[3]+aa[4]*bb[0]+aa[5]*bb[3];
		double a4 = aa[4]*bb[1]+aa[5]*bb[4];
		double a5 = aa[4]*bb[2]+aa[5]*bb[5];

		this->a[0] = a0;
		this->a[1] = a1;
		this->a[2] = a2;
		this->a[3] = a3;
		this->a[4] = a4;
		this->a[5] = a5;
    }
} // void CvTransf::mult( CvTransf &a, CvTransf &b)	// this = a*b.  ,  
////////////////////////////////////////////////////////////////
//void CvTransf::multLeft( CvTransf &b)// this = b*this.  ,  
//{
//	CvTransf t;
//	t.copyFrom(b);
//	t.mult(*this);
//	this->copyFrom(t);
//} // void CvTransf::multLeft( CvTransf &b)// this = b*this.  ,  
//////////////////////////////////////////////////////////////////
void CvTransf::divide( CvTransf &a, CvTransf &b)//  this ,  this*a = b
{
	CvTransf r;
	r.revers(a);
	this->mult(b, r);
} // void CvTransf::divide( CvTransf &a, CvTransf &b)//  this ,  this*a = b

//////////////////////////////////////////////////////////////////
void CvTransf::revers( CvTransf &b)	// this = (b)
{
	kind = b.kind;
	if( kind == tTransf_shift )
	{
		a[0] = -b.a[0];
		a[1] = -b.a[1];
	} 
	else if( kind == tTransf_rotate )
	{
		double b0 = b.a[0];
		double b1 = b.a[1];
		double b2 = b.a[2];
		double co = cos(b2);
		double si = sin(b2);
		a[0] = b1*si - b0*co;
		a[1] = -co*b1-b0*si;
		a[2] = -b2;
	} 
	else //if( kind == tTransf_affine )
	{
		double b0 = b.a[0];
		double b1 = b.a[1];
		double b2 = b.a[2];
		double b3 = b.a[3];
		double b4 = b.a[4];
		double b5 = b.a[5];
		double de = b5*b1 -b4*b2;
		//printf("de=%10.5f\n", de);
		if( de==0 )	// .  
		{
			this->save("prot.txt"); 
			vError("CvTransf::revers: degenerate affine");
			set1();
			kind = tTransf_none;
			return;
		}

		this->a[0] = (b2*b3-b0*b5)/de;
		this->a[1] = b5/de;
		this->a[2] = -b2/de;

		this->a[3] = (b4*b0-b3*b1)/de;
		this->a[4] = -b4/de;
		this->a[5] = b1/de;
    }
}
////////////////////////////////////////////////////////////////
void CvTransf::power( CvTransf &b, int k)	// this = b^k;
{
	if( k<=0 )	
	{
		this->revers(b);
		if( k==-1 ) return;
		// k < -1;
		CvTransf bm;
		bm.copyFrom(*this);
		for( int i=-1; i>k; i-- )
			this->mult(*this, bm);

		return;
	}
	if( k==0 )	
	{
		set1();
		return;
	}

	if( k==1 )	
	{
		this->copyFrom(b);
		return;
	}

	// k>1
	this->copyFrom(b);
	for( int i=1; i<k; i++ )
		this->mult(*this, b);

} // void CvTransf::power( CvTransf &b, int k)	// this = b^k;
////////////////////////////////////////////////////////////////
void CvTransf::setNothing()	//   
{
	this->kind = tTransf_none;
	this->a[0] = 1e99;
}

bool CvTransf::isNothing()	// ,     
{
	if( this->kind == tTransf_none ) return true;
	return false;
}
/////////////////////////////////////////////////////////////////////////////
//       .
//    (cx,cy)     (r)   
//   6-    (buff[0..5]),     .
//    . 
//   : 
// P0 -  ,       (cx  , cy  )
// P1 -    r,    (cx  , cy-r)
// P2 -    r, (cx+r, cy  )
//
//  :
// A         -  ,
// (v0x,v0y) -    P0, (P0)-P0
// (v1x,v1y) - : (   P1)-(   P0),
//			   (A(P1)-P1) - (A(P0)-P0) = A(P1)-A(P0)-P1+P0,
//   (+) A' , 
//
// A'(P0) = A(P0),
// A'(P1) = A(P1).
//
//      .
// 
//
// (v2x,v2y) - A'(P2) - A(P2)
//
//  (v0x,v0y, v1x,v1y, v2x,v2y)    
//   (cx,cy,r).
//
//        16    .
//    buff[0..5]
// 
void CvTransf::code6(double cx, double cy, double r, double *buff)	//    6  
{
	if( Kind() == tTransf_none ) vError("CvTransf::code6: kind = tTransf_none");
	double v0x,v0y, v1x,v1y, v2x,v2y;

	//   v0x,v0y
	this->apply(cx,cy, v0x,v0y);
	v0x -= cx;
	v0y -= cy;

	//  v1x,v1y
	this->apply(cx,cy-r, v1x,v1y);
	v1x -= cx;		
	v1y -= cy-r;	// (v1x,v1y) -     P1
	v1x -= v0x;
	v1y -= v0y;		// 

	//  v2x,v2y
	this->apply(cx+r,cy, v2x,v2y);
	v2x -= cx+r;		
	v2y -= cy;		// (v2x,v2y) -     P2

	v2x -= v0x;
	v2y -= v0y;		

	v2x += v1y;
	v2y -= v1x;

	// .   

	buff[0] = v0x;
	buff[1] = v0y;
	buff[2] = v1x;
	buff[3] = v1y;
	buff[4] = v2x;
	buff[5] = v2y;
} // void CvTransf::code6( int cx, int cy, int r, int *buff )	//    6  
/////////////////////////////////////////////////////////////////////////////
void CvTransf::code6(int cx, int cy, int r, int *buff)	//    6  
{
	int co = 16;	//     
	double dbuf[6];
	code6(cx, cy, r, dbuf);
	buff[0] = round(co*dbuf[0]);
	buff[1] = round(co*dbuf[1]);
	buff[2] = round(co*dbuf[2]);
	buff[3] = round(co*dbuf[3]);
	buff[4] = round(co*dbuf[4]);
	buff[5] = round(co*dbuf[5]);
}
/////////////////////////////////////////////////////////////////////////////
void CvTransf::decode6(int cx, int cy, int r, int *buff) //    6  
{
	double co = 1. / 16;	//     
	double v0x, v0y, v1x, v1y, v2x, v2y;

	v0x = co*buff[0];
	v0y = co*buff[1];
	v1x = co*buff[2];
	v1y = co*buff[3];
	v2x = co*buff[4];
	v2y = co*buff[5];

	double x0, y0, x1, y1, x2, y2;		//  P0, P1, P2

										//    P0:
	x0 = cx + v0x;
	y0 = cy + v0y;

	//   P1:
	x1 = x0 + v1x;
	y1 = y0 - r + v1y;

	//   P2:
	x2 = x0 + r - v1y + v2x;
	y2 = y0 + v1x + v2y;

	this->setAffine(cx, cy, cx, cy - r, cx + r, cy,
		x0, y0, x1, y1, x2, y2);
	//	x0 = a[0] + a[1]*x + a[2]*y;
	//	y0 = a[3] + a[4]*x + a[5]*y;
	this->lowKind();

} // void CvTransf::decode6( int cx, int cy, int r, int *buff ) //    6  
/////////////////////////////////////////////////////////////////////////////
void CvTransf::decode6(double cx, double cy, double r, double *buff) //    6   
{
	if (r < 1) vError("CvTransf::decode6: r=%10.5f", r);
	double v0x, v0y, v1x, v1y, v2x, v2y;

	v0x = buff[0];
	v0y = buff[1];
	v1x = buff[2];
	v1y = buff[3];
	v2x = buff[4];
	v2y = buff[5];

	double de1 = 1 / r;
	kind = tTransf_affine;
	a[0] = de1*(v0x*r + cx*v1y - cx*v2x + v1x*cy);
	a[1] = de1*(r - v1y + v2x);
	a[2] = de1*(-v1x);
	a[3] = de1*(v0y*r - cx*v1x - cx*v2y + cy*v1y);
	a[4] = de1*(v1x + v2y);
	a[5] = de1*(r - v1y);
} //void CvTransf::decode6(double cx, double cy, double r, double *buff) //    6   
/////////////////////////////////////////////////////////////////////////////
char CvTransf_code6Str[512];						//    
char *CvTransf::code6Str( int cx, int cy, int r, bool xls)	// :    6  
{
	if( Kind() == tTransf_none ) vError("CvTransf::code6Str: kind = tTransf_none");
	int buf[6];
	this->code6(cx, cy, r, buf );
	if(xls)
	sprintf(CvTransf_code6Str, "%6.2f\t %6.2f\t %6.2f\t %6.2f\t %6.2f\t %6.2f", 
		buf[0]/16.0, buf[1]/16.0, buf[2]/16.0, buf[3]/16.0, buf[4]/16.0, buf[5]/16.0);
	else
	sprintf(CvTransf_code6Str, "(%6.2f,%6.2f | %6.2f,%6.2f | %6.2f,%6.2f)", 
		buf[0]/16.0, buf[1]/16.0, buf[2]/16.0, buf[3]/16.0, buf[4]/16.0, buf[5]/16.0);
	return CvTransf_code6Str;
}
/////////////////////////////////////////////////////////////////////////////
char *CvTransf::code6Sts( int cx, int cy, int r, bool xls)// code6Str    16
{
	if( Kind() == tTransf_none ) vError("CvTransf::code6Str: kind = tTransf_none");
	int buf[6];
	this->code6(cx, cy, r, buf );
	if(xls)
	sprintf(CvTransf_code6Str, "%6d\t %6d\t %4d\t %4d\t %4d\t %4d", 
		buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]);
	else
	sprintf(CvTransf_code6Str, "(%6d,%6d | %4d,%4d | %4d,%4d)", 
		buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]);
	return CvTransf_code6Str;
}
/////////////////////////////////////////////////////////////////////////////
char *CvTransf::codeSimpl(bool xls)		//  
{
	if( Kind() == tTransf_none ) vError("CvTransf::codeSimpl: kind = tTransf_none");

	CvTransf a;
	a.convertFrom(*this, tTransf_affine);

	if(xls)
	sprintf(CvTransf_code6Str, "%1d\t %7.2f\t %7.4f\t %7.4f\t %7.2f\t %7.4f\t %7.4f", 
		(int)Kind(), a.get(0), a.get(1), a.get(2), a.get(3), a.get(4), a.get(5));
	else
	sprintf(CvTransf_code6Str, "<%1d|%7.2f,%7.4f %7.4f|%7.2f,%7.4f %7.4f>", 
		(int)Kind(), a.get(0), a.get(1), a.get(2), a.get(3), a.get(4), a.get(5));
	return CvTransf_code6Str;
}
/////////////////////////////////////////////////////////////////////////////
void CvTransf::lowKind()	//   ,  
{
	if( Kind() == tTransf_none ) vError("CvTransf::lowKind: kind = tTransf_none");
	if( this->kind == tTransf_shift ) return;

	if( this->kind == tTransf_affine )
	{
		if( fabs(a[1]-1) + fabs(a[2]) + fabs(a[4]) + fabs(a[5]-1)<0.00001 )
		{
			this->kind = tTransf_shift;
			a[1] = a[3];
			a[2] = a[3] = a[4] = a[5] = 0;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  A(x,y) = (a0 + a1*x + a2*y, a3 + a4*x + a5*y) -   
// , 
//	A(x1, y1) = (x1s, y1s), 
//	A(x2, y2) = (x2s, y2s), 
//	A(x3, y3) = (x3s, y3s), 
//
// 
//	a0,a1,a2,a3,a4,a5
void affineTMove(double x1,  double y1,   double x2,  double y2,   double x3,  double y3,  
				 double x1s, double y1s,  double x2s, double y2s,  double x3s, double y3s,  
				 double &a0, double &a1,  double &a2, double &a3,  double &a4, double &a5 )
{
	double d  = x3*y1-x3*y2+x1*y2-x1*y3+x2*y3-x2*y1;
	if( d==0 ) //  .   
		{ a0=a2=a3=a4=0; a1=a5=1; return; }
	a0 = (-y2*x1s*x3+x2s*x3*y1-y3*x1*x2s+y3*x1s*x2+x3s*x1*y2-x3s*x2*y1)/d;
	a1 = (y3*x2s-y3*x1s+x3s*y1-x3s*y2+y2*x1s-x2s*y1)/d;
	a2 = (-x3*x2s+x3s*x2-x1*x3s+x1*x2s+x1s*x3-x1s*x2)/d;
	a3 = (-y3*y2s*x1+y3s*x1*y2-y1*x2*y3s+y1*y2s*x3-y1s*x3*y2+y1s*x2*y3)/d;
	a4 = (y3s*y1-y1*y2s+y1s*y2-y3*y1s+y3*y2s-y3s*y2)/d;
	a5 = (-x1*y3s+y1s*x3-x2*y1s+x2*y3s-y2s*x3+y2s*x1)/d;
}
/////////////////////////////////////////////////////////////////////////////////////
void CvTransf::Transf2shift( CvTransf  &sh, double px, double py )//      (px,py)
{
	if( Kind() == tTransf_none ) vError("Transf2rot: kind = tTransf_none");
	double x1,y1;
	apply( px,py, x1,y1 );
	sh.set( x1-px, y1-py);
}
////////////////////////////////////////////////////////////////////////////////
void CvTransf::Transf2rot  (CvTransf &rot, double px, double py, double r )//     a
{
	if( Kind() == tTransf_none ) vError("Transf2rot: kind = tTransf_none");
	//       list   
	double x,y;
	double x0, y0, x1,y1;
	double vx=0, vy=0;		//    
	if( r==0 ) vError("Transf2rot: r=0");
	//int n = list.getSize();
	//  5 
	const int nP=5;
	double pnt[2*nP] = {
		px, py, 
		px+r, py,
		px-r, py,
		px, py+r,
		px, py-r,
	};
	int i;
	for( i=0; i<2*nP; i+=2 )
	{
		x = pnt[i];
		y = pnt[i+1];
		apply( x,y, x1,y1 );
		vx += x1-x;
		vy += y1-y;
	}
	vx   /= nP;
	vy   /= nP;
	// (vx  ,vy  ) -    
	//     
	double weight=0;	//    ;
	double angleSu=0;	//  
	double co, si;		// cos(..), sin(..)
	for( i=0; i<2*nP; i+=2 )
	{
		x = pnt[i];
		y = pnt[i+1];
		x0 = x - px;
		y0 = y - py;
		// (x0,y0) -     
		apply( x,y, x1,y1 );
		x1 -= px + vx;
		y1 -= py + vy;
		// (x1,y1) -     
		//     (x0,y0)  (x1,y1)
		//   :
		double len = x0*x0 + y0*y0;
		if( len==0 ) continue;
		len = 1/sqrt(len);
		x0 *= len; y0 *= len;
		len = x1*x1 + y1*y1;
		if( len==0 ) continue;
		len = 1/sqrt(len);
		x1 *= len; y1 *= len;
		// :
		// (x0,y0) -      
		// (x1,y1) -      
		// 
		//     (x0,y0)  (x1,y1)
		co = x0*x1 + y0*y1;	//    
		si =-x0*y1 + y0*x1;	//    

		double angl = angle(co, si);
		if( angl > 3.14159265358979324 ) angl = angl - 6.28318530717958648;
		// angle -     -Pi..Pi
		len = 1/len;
		angleSu += angl*len;
		weight  += len;
	}
	angleSu /= weight;

	co = cos(angleSu);
	si = sin(angleSu);
	x0 = px + vx - co*px - si*py;
	y0 = py + vy + si*px - co*py;
	rot.set( x0 , y0, angleSu ); 

} // void Transf2rot  ( CvTransf &a, CvRotate &rot, pvector &list )	//     a
////////////////////////////////////////////////////////////////////////////////
double CvTransf::TransfDist(CvTransf &a2, double px, double py, double r)// .   list  a1(v)  a2(v)
{
	if (   Kind() == tTransf_none ||
		a2.Kind() == tTransf_none) vError("TransfDist: kind = tTransf_none");
	double x, y;
	double x1, y1, x2, y2;
	double s = 0;
	if (r == 0) vError("TransfDist: r=0");
	//  5 
	const int nP = 5;
	double pnt[2 * nP] = {
		px, py,
		px + r, py,
		px - r, py,
		px, py + r,
		px, py - r,
	};
	int i;
	for (i = 0; i<2 * nP; i += 2)
	{
		x = pnt[i];
		y = pnt[i + 1];
		apply(x, y, x1, y1);
		a2.apply(x, y, x2, y2);
		s += dSquare(x2 - x1) + dSquare(y2 - y1);
	}
	s = sqrt(s / nP);
	return s;
}
////////////////////////////////////////////////////////////////////////////////
double CvTransf::TransfMaxOffs(double px, double py, double r)	//      
{
	double x, y;
	double x1, y1;
	double sm = 0, s;

	if (r == 0) vError("TransfMaxOffs: r=0");
	//  5 
	const int nP = 5;
	double pnt[2 * nP] = {
		px, py,
		px + r, py,
		px - r, py,
		px, py + r,
		px, py - r,
	};
	int i;
	for (i = 0; i<2 * nP; i += 2)
	{
		x = pnt[i];
		y = pnt[i + 1];
		apply(x, y, x1, y1);
		s = dSquare(x1 - x) + dSquare(y1 - y);
		if (s>sm) sm = s;
	}
	sm = sqrt(sm);
	return sm;
}
////////////////////////////////////////////////////////////////////////////////
bool   TransBad  (  CvTransf &a, double v )	//  ?
{
	if( a.Kind() == tTransf_none ) return true; 
	else if( a.Kind() == tTransf_shift ) {		//    
		if( (fabs(a.get(0))>v) || (fabs(a.get(1))>v)) return true;
		return false;
	} else if( a.Kind() == tTransf_rotate){		//     
		if( fabs(a.get(2))>1 ) return true;
		return false;
	} else if( a.Kind() == tTransf_affine ){	//    
		if( (fabs(a.get(1))>2) || (fabs(a.get(2))>2) || (fabs(a.get(4))>2) || (fabs(a.get(5))>2)) return true;
		return false;
	} else return true;							//     
}
////////////////////////////////////////////////////////////////////////////////
void Transt_FullTest()		//    CvTransf;
{
	CvTransf a0, a1, a2;
	double x,y, x0,y0, x1,y1, x2,y2;
	double phi;
	double err;

	//  apply:

	x = 777.777;	y = 123.456;
	a0.set( x, y );
	a0.apply( 1,7, x0, y0);
	err = fabs(x+1-x0)+fabs(y+7-y0);
	vShow( " 1. err= %12.9f\n", err);

	phi = 1.0;
	a0.set( x, y, phi );
	a0.apply( 1,1, x0, y0);
	err = fabs(x+cos(phi)+sin(phi)-x0)+fabs(y-sin(phi)+cos(phi)-y0);
	vShow( " 2. err= %12.9f\n", err);

	a0.set( 1, 2, 3, 4, 5, 6 );
	a0.apply( 77, 88, x0, y0);
	err = fabs(1 + 2*77 + 3*88 - x0)+fabs(4 + 5*77 + 6*88-y0);
	vShow( " 3. err= %12.9f\n", err);

	//   :
	x = 777.777;	y = 123.456;
	a0.set( x, y );
	a0.apply( 7,9, x0, y0);
	a1.convertFrom( a0, tTransf_rotate );
	a1.apply( 7,9, x1, y1);
	err = fabs(x1-x0)+fabs(y1-y0);
	vShow( " 4. err= %12.9f\n", err);

	a1.convertFrom( a0, tTransf_affine );
	a1.apply( 7,9, x1, y1);
	err = fabs(x1-x0)+fabs(y1-y0);
	vShow( " 5. err= %12.9f\n", err);

	a0.set( x, y, phi );
	a0.apply( 7,9, x0, y0);
	a1.convertFrom( a0, tTransf_affine );
	a1.apply( 7,9, x1, y1);
	err = fabs(x1-x0)+fabs(y1-y0);
	vShow( " 6. err= %12.9f\n", err);

	//  :
	x = 777.777;	y = 123.456;
	a0.set( x, y );
	a1.revers(a0);
	a0.apply( 7,9, x0, y0);
	a1.apply( x0,y0, x1, y1);
	err = fabs(x1-7)+fabs(y1-9);
	vShow( " 7. err= %12.9f\n", err);

	a0.set( x, y, phi );
	a1.revers(a0);
	a0.apply( 7,9, x0, y0);
	a1.apply( x0,y0, x1, y1);
	err = fabs(x1-7)+fabs(y1-9);
	vShow( " 8. err= %12.9f\n", err);

	a0.set( 1.1, 2.2, 3.7, 4.1, 5.9, 6.2 );
	a1.revers(a0);
	a0.apply( 7,9, x0, y0);
	a1.apply( x0,y0, x1, y1);
	err = fabs(x1-7)+fabs(y1-9);
	vShow( " 9. err= %12.9f\n", err);

	//  : 
	a0.set( 7.7, 9.9 );
	a1.set( 3.7, 1.9 );
	x = 777.777;	y = 123.456;
	a0.apply(x ,y , x0,y0);
	a1.apply(x0,y0, x1,y1);

	a2.mult(a0, a1);
	a2.apply(x ,y , x2,y2);
	err = fabs(x1-x2)+fabs(y1-y2);
	vShow( "10. err= %12.9f\n", err);

	//  : 
	a0.set( 7.7, 9.9, 4.5 );
	a1.set( 3.7, 1.9, 7.1 );
	//a0.set( 7, 4, 2 );
	//a1.set( 3, 1, 1 );
	a2.mult(a1, a0);

	x = 777.777;	y = 123.456;
	a0.apply(x ,y , x0,y0);
	a1.apply(x0,y0, x1,y1);

	a2.apply(x ,y , x2,y2);
	err = fabs(x1-x2)+fabs(y1-y2);
	vShow( "12. err= %12.9f\n", err);

	//  : 
	a0.set( 7.7, 9.9, 4.5, 3.7, 1.9,-7.1 );
	a1.set( 3.7, 1.9, 7.1, 9.9,-4.5, 6.7 );
	//a0.set( 3, 1, 0, 4, 0, 1 );
	//a1.set( 1, 1, 0, 2, 0, 1 );
	a2.mult(a1, a0);

	x = 777.777;	y = 123.456;
	a0.apply(x ,y , x0,y0);
	a1.apply(x0,y0, x1,y1);

	a2.apply(x ,y , x2,y2);
	err = fabs(x1-x2)+fabs(y1-y2);
	vShow( "14. err= %12.9f\n", err);

	//  setAffine
	x = 777.777;	y = 123.456;
	x0= 317.379;	y0=  24.111;
	x1= 117.469;	y1= -33.263;
	a0.setAffine(x,y, x0,y0, x1,y1, 5,5, 5,6, 6,5);
	a0.apply(x,y, x2,y2);
	err = fabs(5-x2)+fabs(5-y2);
	a0.apply(x0,y0, x2,y2);
	err+= fabs(5-x2)+fabs(6-y2);
	a0.apply(x1,y1, x2,y2);
	err+= fabs(6-x2)+fabs(5-y2);
	vShow( "16. err= %12.9f\n", err);

	//  power
	a1.power(a0,4);
	a0.apply(x,y, x0,y0);		// a0(x,y)
	a0.apply(x0,y0, x0,y0);		// a0^2(x,y)
	a0.apply(x0,y0, x0,y0);		// a0^3(x,y)
	a0.apply(x0,y0, x0,y0);		// a0^4(x,y)
	a1.apply(x ,y , x1,y1);
	err = fabs(x1-x0)+fabs(y1-y0);
	vShow( "17. err= %12.9f\n", err);

	//  (de)code6
	fileClear("res.txt");
	int buff[6];
	int cx=100, cy=100, r=10;
	//a0.set( 11.375, -7.125, 0.1 );
	//a0.set( 5.6789, -1.2345 );
	//a0.set( 5.6789, -1.2345, 0.1111 );
	a0.set( 16.081371, 0.918661, 0.012115, 25.397842, -0.141696, 0.940863);

	a0.  code6( cx,cy, r, buff);
	a1.decode6(cx, cy, r, buff);
	a0.save( "res.txt");
	a1.save( "res.txt");
	a0.apply( 107.777,  93.456, x0,y0);
	a1.apply( 107.777,  93.456, x1,y1);
	//err = fabs(x1-x0)+fabs(y1-y0);
	err = hypot(x1-x0, y1-y0);
	vShow( "18. 16*err= %12.9f  (must be <1)\n", 16*err);

	//  divide
	a0.set( 7.7, 9.9, 4.5, 3.5, 1.9,-7.1 );
	a1.set( 3.7, 1.6, 7.1, 9.7,-4.4, 6.7 );
	a2.divide(a0,a1);				//   a2*a0 = a1

	x = 777.777;	y = 123.456;
	a0.apply(x ,y , x0,y0);
	a2.apply(x0,y0, x2,y2);

	a1.apply(x ,y , x1,y1);
	err = fabs(x1-x2)+fabs(y1-y2);
	vShow( "19. err= %12.9f\n", err);

	vError("OK");
}
////////////////////////////////////////////////////////////////////////////////
double CvTransf::dist(CvTransf &tr2, double cx, double cy, double r)	//   this  tr2     (cx,cy)   r
{
	double x0, y0, x2, y2;
	double sum2 = 0;

	apply(cx + r, cy, x0, y0);
	tr2.apply(cx + r, cy, x2, y2);
	sum2 = dSquare(x2 - x0) + dSquare(y2 - y0);

	apply(cx - r, cy, x0, y0);
	tr2.apply(cx - r, cy, x2, y2);
	sum2 += dSquare(x2 - x0) + dSquare(y2 - y0);

	apply(cx, cy + r, x0, y0);
	tr2.apply(cx, cy + r, x2, y2);
	sum2 += dSquare(x2 - x0) + dSquare(y2 - y0);

	apply(cx, cy - r, x0, y0);
	tr2.apply(cx, cy - r, x2, y2);
	sum2 += dSquare(x2 - x0) + dSquare(y2 - y0);

	return sqrt(sum2/4);
}


////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
