// vUtil.cpp 2004-03-26: general functions and data
// (c) Khashin S.I. khash2@gmail.com http://math.ivanovo.ac.ru/dalgebra/Khashin/index_r.html
//
//////////////////////////////////////////////////////////////////////
#ifdef _MSC_VER
  #pragma warning(disable : 4996)
#endif

#ifdef __GNUC__
#pragma GCC diagnostic ignored "-Wwrite-strings"
#pragma GCC diagnostic ignored "-Wformat"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <direct.h>
#include <sys/timeb.h>
#include <time.h>

#ifdef _MSC_VER
#include <Windows.h>				// only for Windows!
#include <intrin.h>					// Compiler Intrinsics
#endif


#include "vUtil.h"

void    vError(const char *format, ...) // show error function
{
	va_list argp;
	va_start(argp, format);
	fprintf( stderr, "\nvError: ");
	vfprintf(stderr, format, argp);
	va_end(argp);
	fprintf( stderr, "\nPress <Enter>\n");
	getchar();
	_exit(1);
}

void    vShow (const char *format, ...) // show function
{
	va_list argp;
	va_start(argp, format);
	vprintf(format,argp);
	//printf( "\n");
	va_end(argp);
}
//-----------------------------------------------------------------------------
//   fname,     msg   
void	hWrite0(const char *fname, const char *msg)
{
	FILE *f = fopen(fname, "a");
	if (!f) vError("hWrite: file %s not open", fname);
	fprintf(f, msg);
	fclose(f);
}

void	hWrite(const char *fname, const char *format, ...)
{
	if (strlen(fname) == 0) return;
	FILE *f = fopen(fname, "a");
	if (!f) vError("hWrite: file %s not open", fname);
	va_list argp;
	va_start(argp, format);
	vfprintf(f, format, argp);
	va_end(argp);
	//fprintf(f, "\n");
	fclose(f);
}
//-----------------------------------------------------------------------------
// string funtions
u64  atou64(const char *s)		//    VS2010  atoll
{
	u64 res=0;
	while('0' <= *s && *s <= '9' ) {
		res = res*10+(*s-'0');
		s++;
	}
	return res;
}
//-----------------------------------------------------------------------------
char toString_buf[512];
char *	int2str(char* format, int x ) // x.toString()
{
	sprintf( toString_buf, format, x );
	return toString_buf;
}

char *	toString(const char* format, ... ) // x.toString()
{
	va_list argp;
	va_start(argp, format);
	vsprintf(toString_buf, format, argp);
	va_end(argp);
	//char *tmp = new char[ strlen(toString_buf)+1];
	//strcpy(tmp, toString_buf);
	//return tmp;
	return toString_buf;
}

/////////////////////////////////////////////////////////////////////////
//  toString    ,  
//
// printf("%s_%s", toString(x), toString(y) )
//
// ,           .
//   ,  .     :
// 1)       ,  
//     new.           .
// 2)    ,    () .
//     , ,     ,
//
//    printf("%s_%s_%s", toString(x), toString(y), toString(z) )
//
//        ,      .
char toString_buf2[512];
char *	toString2(const char* format, ... ) // x.toString()
{
	va_list argp;
	va_start(argp, format);
	vsprintf(toString_buf2, format, argp);
	va_end(argp);
	//char *tmp = new char[ strlen(toString_buf)+1];
	//strcpy(tmp, toString_buf);
	//return tmp;
	return toString_buf2;
}
///////////////////////////////////////////////////////////////
char *	copyString( const char *s ) //    
{
	char *s1 = new char[ strlen(s)+1 ];
	strcpy( s1, s );
	return s1;
}
//----------------------------------------------------------------------------
//   s         '\0'.
//    ,    '\0'.
//    ,      NULL.
char *nextToken( char *s )
{
	if( s==NULL ) return NULL;
	char *s1 = strchr(s, '\t');
	if( s1==NULL ) return NULL;
	*s1++ = 0;
	return s1;
}
//----------------------------------------------------------------------------
const char *FirstWord(const char *src, const char *brk, char *dst)	// dst:=1- ,    brk,     
{
	size_t len = strspn(src, brk);		//    ,     brk
	src+=len;							// src     
	len = strcspn(src, brk);			//   ,     brk,    
	strncpy(dst, src, len);
	dst[len] = 0;
	return src + len;
}
//----------------------------------------------------------------------------
int  chrKind(char ch)
{
	if( ch>='0' && ch<='9' ) return chrKind_DIGIT;
	if( ch>='' && ch<='' ) return chrKind_RUS;
	if( ch>='A' && ch<='Z' ) return chrKind_LAT;
	if(ch >='a' && ch<='z' ) return chrKind_LAT;
	if (isPnt(ch)) return chrKind_PNT;
	return chrKind_REST;
}
//----------------------------------------------------------------------------
bool isOnlylRus( char *s)		//      ?
{
	while(*s)
		if( !islRus(*s++) ) return false;
	return true;
}
//-----------------------------------------------------------------------------
void ltrim(char *s) //     
{
	char *s1 = s;
	while( (*s1==' ') || (*s1=='\t')) s1++;
	if( s==s1 ) return;
	while(*s1) { *s++ = *s1++; }
	*s=0;
}
//-----------------------------------------------------------------------------
void rtrim(char *s) //     
{
	size_t k = strlen(s)-1;
	while( k>=0 && (s[k]==' '|| s[k]=='\t'|| s[k]=='\n'|| s[k]=='\r')) k--;
	s[k+1]=0;
}
//-----------------------------------------------------------------------------
void trim(char *s) //       
{
	rtrim(s);
	ltrim(s);
}
//-----------------------------------------------------------------------------
unsigned char  char_866_1251(unsigned char ch)		// dos    -> win
{
	if (ch < 128) return ch;
	if (128 <= ch && ch <= 175) return ch + 64; // ''..''
	if (224 <= ch && ch <= 239) return ch + 16; // ''..''
	if (240 == ch) return (unsigned char)168;	// 
	if (241 == ch) return (unsigned char)184;	// 
	return 0;
}
//-----------------------------------------------------------------------------
unsigned char  char_Koi8_1251(unsigned char ch)	// koi8-r -> win
{
	unsigned char su[257] = "                                 !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~??????????????????????????????????????????????????????????";
	return su[ch];
}
//-----------------------------------------------------------------------------
unsigned char  char_utf8_1251(unsigned char *&s)// uft8   -> win,   , s0    
{
	//       :
	//   hex            
	//   0..127  00-7f       1
	// 128..191  80-bf           
	// 192..223  c0-df       2
	// 224..239  e0-ef       3
	// 240..247  f0-f7       4
	// 248..251  f8-fb       5
	// 252..253  fc-fd       6
	// 254..255  fe-ff           
	//  ,        128  191,      10
	//  6    


	//int c0, c1, v;
	//c0 = *s++;
	//if (c0 < 192 || c0 >= 224) return 0;
	//c1 = *s;
	//if (c1 < 128 || c1 >= 192) return 0;
	//s++;									//   !
	//v = (c0 - 192) * 64 + (c1 - 128);
	//if (1040 <= v && v <= 1103) return (char)(v - 1040 + 192);
	//if (v == 1025) return '';
	//if (v == 1105) return '';
	////   ,  c0;
	//return 0;

	int c0, c1, v;
	c0 = *s++;
	if (c0 < 128) return c0;				// ascii
	if (c0 < 192 || c0 >= 224) return ' ';	//  
	c1 = *s;
	if (c1 < 128 || c1 >= 192) return ' ';
	s++;									//   !
	v = (c0 - 192) * 64 + (c1 - 128);
	if (1040 <= v && v <= 1103) return (char)(v - 1040 + 192);
	if (v == 1025) return '';
	if (v == 1105) return '';
	//   ,  ;
	return ' ';
}
//-----------------------------------------------------------------------------
unsigned char* cnv1251_866(unsigned  char *s )					// win -> dos
{
	if( s==NULL ) return NULL;
	unsigned char *s0 = s;
	unsigned char ch;
	while(*s)
	{
		ch = *s;
		if (ch >= 192 && ch <= 239) *s = ch - 64; else  // if( ch>='' && ch<='' ) *s = ch-64; else
		if (ch >= 240 && ch <= 255) *s = ch - 16; else  // if (ch >= '' && ch <= '') *s = ch - 16; else
		if (ch == 168) *s = 240; else                   // if (ch == '') *s = (240 - 256); else
		if (ch == 184) *s = 241;                        // if( ch=='' ) *s = (241-256);
		s++;
	}
	return s0;
}
//-----------------------------------------------------------------------------
unsigned char* cnv866_1251(unsigned  char *s)					// dos -> win
{
	if (s == NULL) return NULL;
	unsigned char *s0 = s;
	unsigned char ch;
	while (*s)
	{
		ch = *s;
		if (ch >= 128 && ch <= 175) *s = ch + 64; else  // if( ch>='' && ch<='' ) *s = ch+64; else
		if (ch >= 224 && ch <= 239) *s = ch + 16; else  // if (ch >= '' && ch <= '') *s = ch - 16; else
		if (ch == 240) *s = 168; else                   // if (ch == '') *s = (240 - 256); else
		if (ch == 241) *s = 184;                        // if( ch=='' ) *s = (241-256);
		s++;
	}
	return s0;
}
//----------------------------------------------------------------------------
unsigned char* cnvKoi8_1251(unsigned char *s)	// koi8-r -> win  (in place)
{
	unsigned char *s0 = s;
	unsigned char su[257] = "                                 !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~??????????????????????????????????????????????????????????";
	for (; *s; s++) {
		if(*s>127)
		*s = su[(unsigned char)(*s)];
	}
	return s0;
}
//----------------------------------------------------------------------------
unsigned char* cnv_utf8_1251(unsigned char *s)// uft8 -> win (in place)
{
	// s[i1] -   
	// s[i0] -   
	// i0 <= i1
	int i0=0, i1=0, c0, c1, v;
	do
	{
		c0 = s[i1];
		if( c0==0 ) break;
		if (c0<128) { s[i0++] = s[i1++]; continue; }					// ascii
		if (c0 < 192) { i1++; continue; }								//  , 
		if (c0 >= 224) { s[i0++] = ' ';  i1+=2; continue; }				//  ,  
		c1 = s[i1+1];
		if( c1==0 ) break;
		if (c1 < 128 || c1 >= 192) { i1+=2; continue; }
		v = (c0-192)*64 + (c1-128);
		if( 1040<=v && v<=1103 ) { s[i0++] = (v-1040+192); i1+=2; }
		else if( v==1025       ) { s[i0++] = ''; i1+=2; }
		else if( v==1105       ) { s[i0++] = ''; i1+=2; }
		else { s[i0++]=' '; i1+=2; }	//  -  ,   
	} while(1);
	s[i0]=0;
	return s;
}
//----------------------------------------------------------------------------
static int iRusLetter(unsigned char ch) {		//      cp-1251
	if (192 <= ch && ch <= 223) return ch - 192;	// ..
	if (224 <= ch && ch <= 255) return ch - 224;	// ..
	if (168 == ch || 184 == ch) return 5;			// 
	return -1;
}
static unsigned freq_r[32] = {		//    ..
	5095, 1380, 3509, 1224, 2477, 5259,  833, 1375, 3290,  286, 2746, 3310, 1851, 4925, 6699, 2261,
	3654, 3746, 4425, 1668,  157,  496,  311, 1188,  616,  243,   22,  765,  429,  286,  219,  791
};	//  = 65536,   = 238614370, sqrt = 15447.14763

//    
//#define rCP_unknown	0
//#define rCP_1251		1
//#define rCP_866		2
//#define rCP_koi8r		3
//#define rCP_utf8		4
int rCodePage(unsigned char *s)//    
{
	double f866, f1251, fkoi8r, futf8;
	f866 = f1251 = fkoi8r = futf8 = 0;
	unsigned cnt = 0;
	unsigned char *s1 = s;
	int k;
	do {
		if (*s1 < 128) continue;
		k = iRusLetter(*s1); if (k >= 0) f1251 += freq_r[k];
		k = iRusLetter(char_866_1251(*s1)); if (k >= 0) f866 += freq_r[k];
		k = iRusLetter(char_Koi8_1251(*s1)); if (k >= 0) fkoi8r += freq_r[k];
		cnt++;
	} while (*s1++);
	f1251 /= cnt;
	f866 /= cnt;
	fkoi8r /= cnt;
	//  utf8
	s1 = s;
	cnt = 0;
	do {
		if (*s1 < 128) { s1++;  continue; }
		k = iRusLetter(char_utf8_1251(s1)); if (k >= 0) futf8 += freq_r[k];
		cnt++;
	} while (*s1);
	futf8 /= cnt;
	//    f866, f1251, fkoi8r, futf8;
	//printf("%6.1f %6.1f %6.1f %6.1f\n", f1251, f866, fkoi8r, futf8);
	if (f1251 <2000   && f866 <2000 && fkoi8r <2000 && futf8 <2000) return rCP_unknown;
	if (f1251 >= f866   && f1251 >= fkoi8r && f1251 >= futf8) return rCP_1251;
	if (f866  >= fkoi8r && f866  >= futf8) return rCP_866;
	if (fkoi8r>= futf8) return rCP_koi8r;
	return rCP_utf8;
}
//----------------------------------------------------------------------------
char* toLatLow(char *txt)		//    
{
	char *s0 = txt;
	for (; *txt; txt++) {
		char ch = *txt;
		if (ch >= 'A' && ch <= 'Z') {
			*txt = ch+32;
		}
	}
	return s0;
}
//----------------------------------------------------------------------------
char* toRusLow(char *txt)	//         
{
	char *s0 = txt;
	unsigned char su[257] = "                                                                                                                                                                                              ";
	for (; *txt; txt++) 
		*txt = su[(unsigned char)(*txt)];
	return s0;
}
char* toRusLowP(char *txt)		//       ,   
{
	char *s0 = txt;
	unsigned char su[257] = "                                 .            .                .                                                                                                                              ";
	for (; *txt; txt++)
		*txt = su[(unsigned char)(*txt)];
	return s0;
}

//----------------------------------------------------------------------------
char* txtPress(char *txt)	// trim +    
{
	char *s0 = txt;
	char *s1, *s2;
	for(s2=txt; isBlank(*s2) || isPnt(*s2); s2++ );	// s2 -      
	if(!*s2) {*txt=0; return s0; }			//     ,   
	// s2 -   
	txt[0] = *s2++;							//       
	for (s1 = txt+1; *s2; s2++)
	{
		if( isBlank(*s2) ) {				//    - 
			if (s1[-1] == ' ') continue;	//     ,    
			if (s1[-1] == '.') continue;	//    ,          
			*s1++ = ' ';					//         (s1)
		}
		else
		{
			if (isPnt(*s2)) {				//    - 
				if (s1[-1] == ' ') {		//    ,     
					s1[-1] = '.'; continue;
				};
				if (s1[-1] == '.') continue;//    ,     
				*s1++ = '.';				//        (s1)
			}
			else
				*s1++ = *s2;
		}
	}
	*s1=0;
	if( isBlank(s1[-1])) s1[-1]=0;
	return s0;
}

//-----------------------------------------------------------------------------
// int funtions

int		iDivide( int a, int b ) // round(a/b)
{
	int sign=1;
	if( a<0 ) {a=-a; sign=-1; }
	int d = (a+b/2)/b;
	return sign>0 ? d : -d;
}

int	 mod( int a, int b )			// math: a mod b
{
	return (a>=0? a%b : b+a%b );
}

int		iround( int x, int k )  //     k
{
	int sign=1;
	if( x<0 ) {x=-x; sign=-1; }
	int d = ((x+k/2)/k)*k;
	return sign>0 ? d : -d ;

}
int		iSquare(int    x) { return x*x; };
BYTE	clip256(int x) { return x<0 ? 0 : x>255 ? 255 : x; }

int		ilog2(unsigned x)			//   2, <= x
{
#ifdef _MSC_VER
	unsigned long ind;
	_BitScanReverse(&ind, (unsigned long)x);
	return ind;
#else
	int p;
	for (p = 0; x>1; x /= 2) p++;
	return p;
#endif
}
int		ilog2(u64 x)				//   2, <= x, ilog2(0)=ilog2(1)=0, ilog2(2)=ilog2(3)=1, ...
{
#ifdef _WIN64
	unsigned long ind;
	_BitScanReverse64(&ind, x);
	return ind;
#else
	int p;
	for (p = 0; x>1; x /= 2) p++;
	return p;
#endif
}
void	swapInt(int &a, int &b) { int tmp = a; a = b; b = tmp; };
/////////////////////////////////////////////////////
u64 binomial( int n, int k )	// n! / (k!*(n-k)!, binomial(3,2)=3
// Be carefull!
// binomial(62,31) = 465 428 353 255 261 088 (correct)
// binomial(63,31) = 148 894 686 314 746 622 (wrong, correct value 916 312 070 471 295 267 too large!)
{
	if( n<1 ) return 0;
	if( k<0 || k>n ) return 0;
	if( k==0 || k==n ) return 1;
	// now 0<k<n
	int t=n-k; if(t<k) k=t;
	// now 0<2k<=n
	//
	//       n*(n-1)*...*(n-k+1)
	// res = ------------------
	//       1*  2  *...*   k
	//
	u64 res = n;
	for( unsigned i=2, j=n-1; (int)i<=k; i++, j-- )
		res=res*j/i;
	return res;
}
//**********************************************************
//
//Description: map signed x to positive integer
//             0,1,2, ... -> 0, 2, 4, ...
//             -1,-2, ... -> 1, 3, ....
//Arguments: signed x
//Return value: appropriate positive value
//
//**********************************************************
int toNatural(int x)
{
    return ((x>=0)?2*x:-1-2*x);
}

//**********************************************************
//
//Description: remap positive x to signed integer
//             0, 2, 4, ... -> 0, 1, 2, ...
//             1, 3, 5, ... -> -1,-2,-3, ...
//Arguments: positive integer
//Return value: appropriate signed value
//
//**********************************************************
int fromNatural(int x)
{
    return ((x&1)?(-1-x)/2:x/2);
}
//////////////////////////////////////////////////////////////////////
int     imax2( int a, int b ) { return a<b? b : a; }
int     imax3( int a, int b, int c ) { return a<b? (b<c? c : b) : (a<c? c : a); }
int     imax4( int a, int b, int c, int d) {
	if (d >= a && d >= b && d >= c) return d;
	return imax3(a, b, c);
}

int     imin2( int a, int b ) { return a>b? b : a; }
int     imin3( int a, int b, int c ) { return a>b? (b>c? c : b) : (a>c? c : a); }
int     imin4(int a, int b, int c, int d) {
	if (d <= a && d <= b && d <= c) return d;
	return imin3(a, b, c);
}

//////////////////////////////////////////////////////////////////////
int     imax(int N, int *a, int &k)	//    a[0..N-1], k -    
{
	int i, ma = -2147483647;
	for (i = 0, k = -1; i<N; i++)
		if (a[i]>ma)
		{
			ma = a[i];
			k = i;
		}
	return ma;
}
int     imin(int N, int *a, int &k)	//     a[0..N-1], k -    
{
	int i, mi = 2147483647;
	for (i = 0, k = -1; i<N; i++)
		if (a[i]<mi)
		{
			mi = a[i];
			k = i;
		}
	return mi;
}
void corrMinMax(int x, int &min, int &max) {
	if (x < min) min = x;
	if (x > max) max = x;
}
//////////////////////////////////////////////////////////////////////
void	iShiftR(int N, int *a)			//   : a[0]->a[1]->...->a[N-1]->a[0]
{
	int t = a[N - 1];
	for (int i = N - 1; i > 0; i--) a[i] = a[i - 1];
	a[0] = t;
}
void	iShiftL(int N, int *a)			//    : a[0]<-a[1]<-...<-a[N-1]<-a[0]
{
	int t = a[0];
	for (int i = 0; i <N - 1; i++) a[i] = a[i + 1];
	a[N - 1] = t;
}
//////////////////////////////////////////////////////////////////////
int		byteAdd(int a, int b)			//    
{
	int r;
	unsigned char *ca = (unsigned char *)&a;
	unsigned char *cb = (unsigned char *)&b;
	unsigned char *cr = (unsigned char *)&r;
	for (int i = 0; i < 4; i++) {
		int x = *ca++ + *cb++;
		if (x >= 256)x = 255;
		*cr++ = x;
	}
	return r;
}
//--------------------------------------------------------------------
//     hist[0..k-1]   s
// ,    s  >= p
//       <p,   -1
int  quantil(int k, unsigned *hist, int p) //   hist  k   p
{
	int i, s = 0;
	for (i = 0; i<k; i++)
	{
		s += hist[i];
		if (s >= p) return i;
	}
	return -1;
}
//     hist[0..k-1]   s
// ,    s  >= p
//       <p,   -1
int     quantilr(int k, unsigned *hist, int p) //   hist  k    p
{
	int i, s = 0;
	for (i = k - 1; i >= 0; i--)
	{
		s += hist[i];
		if (s >= p) return k - i;
	}
	return -1;
}
//-----------------------------------------------------------------------------
bool even64(u64 x) { return (x & 1) == 0; }
bool odd64(u64 x) { return (x & 1) == 1; }
//-----------------------------------------------------------------------------
// ldouble funtions
ldouble	dSquare(ldouble x) { return x*x;};

ldouble noise2psnr( ldouble x )		//  20*log10(255/x)
{
	if( x==0 ) return 99;
	return 20*log10(255/x);
}

ldouble psnr2dfd( ldouble x )			// 255/exp(0.1151*x)
{
	return 255/exp(0.1151*x);
}

//----------------------------------------------------------------------------
void  swapDbl( ldouble &a, ldouble &b)
{
	ldouble tmp=a; a=b; b=tmp;
}

//////////////////////////////////////////////////////////////////////
ldouble  angle(ldouble x, ldouble y)		//  (0..2Pi)  (x,y)   X.
{
	ldouble angle;
	ldouble len = x*x + y*y;
	if (len == 0) return 0;
	len = 1 / sqrt(len);
	x *= len;
	y *= len;
	//    1.
	// x = cos(angle);
	// y = sin(angle);
	if (x>0 && x>fabs(y))
	{
		angle = asin(y);
		if (angle<0) angle += TwoPi;
		return angle;
	}

	if (x<0 && (-x)>fabs(y))
	{
		angle = -asin(y) + dPi;
		return angle;
	}

	if (y>0)
	{
		return acos(x);
	}

	// if( y<0 ):
	return -acos(x) + TwoPi;
}
//////////////////////////////////////////////////////////////////////
ldouble  hypot2(ldouble x1, ldouble x2) { return  sqrt(x1*x1 + x2*x2); }
ldouble  hypot3(ldouble x1, ldouble x2, ldouble x3) { return  sqrt(x1*x1 + x2*x2 + x3*x3); }
//////////////////////////////////////////////////////////////////////

ldouble  dmin2(ldouble a, ldouble b) { return a>b ? b : a; };
ldouble  dmin3(ldouble a, ldouble b, ldouble c) { return a>b ? (b>c ? c : b) : (a>c ? c : a); }
ldouble  dmin4(ldouble a, ldouble b, ldouble c, ldouble d) {
	if (d <= a && d <= b && d <= c) return d;
	return dmin3(a, b, c);
}

ldouble  dmax2(ldouble a, ldouble b) { return a<b ? b : a; };
ldouble  dmax3(ldouble a, ldouble b, ldouble c) { return a<b ? (b<c ? c : b) : (a<c ? c : a); }
ldouble  dmax4(ldouble a, ldouble b, ldouble c, ldouble d) {
	if (d >= a && d >= b && d >= c) return d;
	return dmax3(a, b, c);
}
//////////////////////////////////////////////////////////////////////
#define SWAP(a,b) { t=b; b=a; a=t; }
void  dblSort3(ldouble &a, ldouble &b, ldouble &c) // make: a <= b <= c
{
	ldouble t;
	if (a>b) SWAP(a, b);	// now a<=b
	if (c<b) {
		SWAP(b, c);			// now a<=b, b<=c
		if (a>b) SWAP(a, b);// now a<=b
	}
}
//////////////////////////////////////////////////////////////////////
bool    isMultiple(ldouble a, ldouble b, ldouble eps)	// a -  b    eps
{
	int k = round(a / b);
	return fabs(a - k*b) < eps;
}
/////////////////////////////////////////////////////////////////////////////////////
int     nDigits(ldouble x)	//    
{
	ldouble eps = 1e-12;
	for( int i=0; i<=8; i++ ) {
		ldouble d = fabs(x-round(x));
		if(d<eps) return i;
		eps *= 10;
		x *= 10;
	}
	return 16;
}
/////////////////////////////////////////////////////////////////////////////////////
void   vdbl_set(int N, ldouble *a, ldouble k)				// a[i] = k --------------------
{
	for (int i = N; i>0; i--) (*a++) = k;
}
void  vdbl_add(int N, ldouble *a, ldouble *b)				// A += B ----------------------
{
	for (int i = N; i>0; i--) (*a++) += (*b++);
}
void   vdbl_cop(int N, ldouble *a, ldouble *b)			// A := B ----------------------
{
	for (int i = N; i>0; i--) (*a++) = (*b++);
}
void  vdbl_mul(int N, ldouble *a, ldouble k)				// A *= k ----------------------
{
	for (int i = N; i>0; i--) (*a++) *= k;
}
void  vdbl_div(int N, ldouble *a, ldouble k)				// A /= k ----------------------
{
	for (int i = N; i>0; i--) (*a++) /= k;
}
void   vdbl_addmul(int N, ldouble *a, ldouble *b, ldouble k)// A += k*B
{
	for (int i = N; i>0; i--) (*a++) += k*(*b++);
}
ldouble vdbl_len2(int N, ldouble *a)			// |A|^2
{
	ldouble res = 0;
	for (int i = N; i>0; i--)
	{
		ldouble d1 = (*a++);
		res += d1*d1;
	}
	return res;
}
ldouble vdbl_len(int N, ldouble *a)			// |A|
{
	ldouble res = 0;
	for (int i = N; i>0; i--)
	{
		ldouble d1 = (*a++);
		res += d1*d1;
	}
	return sqrt(res);
}

void vdbl_norm(int N, ldouble *a)			// |A| := 1
{
	ldouble len = vdbl_len(N, a);
	if (len == 0) return;
	vdbl_mul(N, a, 1 / len);
}

ldouble vdbl_dist2(int N, ldouble *a, ldouble *b)			// |A-B|^2 --------------------
{
	ldouble res = 0;
	for (int i = N; i>0; i--)
	{
		ldouble d1 = (*a++) - (*b++);
		res += d1*d1;
	}
	return res;
}
ldouble vdbl_dist(int N, ldouble *a, ldouble *b)			// |A-B| ----------------------
{
	ldouble res = 0;
	for (int i = N; i>0; i--)
	{
		ldouble d1 = (*a++) - (*b++);
		res += d1*d1;
	}
	return sqrt(res);
}
ldouble  vdbl_sprod(int N, ldouble *a, ldouble *b)			// scalar product(A,B) -------
{
	ldouble res = 0;
	for (int i = N; i>0; i--)
	{
		res += (*a++)*(*b++);
	}
	return res;
}
//---------------------------------------------
ldouble vdbl_max(int N, ldouble *a, int  &ind)		// maximal value and its index
{
	ldouble res = -1e99; ind = -1;
	for (int i = 0; i<N; i++)
	{
		if (a[i]>res) { res = a[i]; ind = i; }
	}
	return res;
}
ldouble vdbl_min(int N, ldouble *a, int  &ind)		// minimal value and its index
{
	ldouble res = 1e99; ind = -1;
	for (int i = 0; i<N; i++)
	{
		if (a[i]<res) { res = a[i]; ind = i; }
	}
	return res;
}
//----------------------------------------------------------------------------
void vdbl_maxK(int N, ldouble *a, int K, int *I) { //   a  N      I  K  
	for (int i = 0; i < K; i++) I[i] = -1;
	int ik = 0;
	ldouble mBord = -dbl_MAX;					//      ,  
	for (int i = 0; i < N; i++) {
		ldouble ai = a[i];
		if (ik < K) {							//   ,     
			I[ik++] = i;						//     
			if (ai > mBord) mBord = ai;
			continue;
		}

		if (ai < mBord) continue;
		//  ai>= mBord
		//   ai>=mBord   
		if (ai == mBord) continue;				//    ,   
												//   ai>mBord    (ik==K),   -  , 
		int jMin = -1;
		ldouble aMin = dbl_MAX;
		for (int j = 0; j < K; j++) {
			double aj = a[I[j]];
			if (aj < aMin) {
				jMin = j;
				aMin = aj;
			}
		}
		//  a[I[jMin]] -  ,   
		I[jMin] = i;
	}
}

//////////////////////////////////////////////////////////////////////
// Continued fraction
bool chainFractStep(ldouble x, i64 &p, i64 &q, i64 maxQ) // one step of chainFract
{
	i64 maxN1;
	if (q <= 0) { q = 1; p = round(x); return true; }
	maxN1 = maxQ / q;
	ldouble err = q*x - p;
	if (fabs(err)*maxQ <1) return false;	//  
	i64 n1 = round(1 / err);
	if (abs(n1)> maxN1) return false; 	//  
	p = p*n1 + 1;
	q *= n1;
	if (n1<0) { q = -q; p = -p; }
	return true;
}
ldouble chainFract(ldouble x, i64 &p, i64 &q, i64 maxQ) // rat.approx p/q of x with precision eps. return tolerance
{
	//ldouble err = 1;
	for (q = 0; chainFractStep(x, p, q, maxQ); );
	//	printf("%d / %d,       %8.2e\n", p, q, abs(x-(ldouble)p/q) );
	return fabs(x - (ldouble)p / q);
}
/////////////////////////////////////////////////////////////////////////
bool	fileExist(const char *fileName)	//    
{
	FILE *f = fopen(fileName, "r");
	if (!f) return false;
	fclose(f);
	return true;
}
const char *fileExt(const char *fileName)  // 
{
	return strrchr(fileName, '.');
}
void	fileClear(const char *fileName)	//  
{
	FILE *f = fopen(fileName, "w");
	fclose(f);
}
int		fileSize(const char *fileName)	//  
{
	FILE *f = fopen(fileName, "rb");
	if (!f) return -1;
	fseek(f, 0, SEEK_END);
	int size = ftell(f);
	fclose(f);
	return size;
}
int		NLines(const char *fileName)		//     
{
	FILE *f = fopen(fileName, "r");
	if (!f) return -1;
	const int sz = 1 << 20;
	char *buf = new char[sz];
	int nl = 0;
	while (!feof(f)) {
		fgets(buf, sz, f);
		nl++;
	}
	delete[]buf;
	fclose(f);
	return nl;
}
int		NLines1(const char *fileName)		//      
{
	FILE *f = fopen(fileName, "r");
	if (!f) return -1;
	const int sz = 1 << 20;
	char *buf = new char[sz];
	int nl = 0;
	while (!feof(f)) {
		fgets(buf, sz, f);
		trim(buf);
		if (strlen(buf)>0) nl++;
	}
	delete[]buf;
	fclose(f);
	return nl;
}

FILE   *aopen(const char *fname)			//  ( )    
{
	FILE *f = fopen(fname, "a");
	if (!f)
		f = fopen(fname, "w");
	return f;
}

bool writeFile(const char *fname, const BYTE *buf, unsigned bsize) {
	FILE *f = fopen(fname, "wb");
	if (!f) return false;
	fwrite(buf, 1, bsize, f);
	fclose(f);
	return true;
}
BYTE * fileRead(const char *fname, unsigned &fsize)	//      (      ).
{
	FILE *f = fopen(fname, "rb");
	if (!f) return NULL;
	fseek(f, 0, SEEK_END);
	fsize = ftell(f);

	BYTE *buf = new BYTE[fsize + 1];		//    
	fseek(f, 0, SEEK_SET);
	fread(buf, 1, fsize, f);
	fclose(f);
	buf[fsize] = 0;
	return buf;
}

/////////////////////////////////////////////////////////////////////////////
//         1
void	nextFname(char *s)		//   
{
	int p = (int)strlen(s) - 1;							// p -     
	while (p >= 0 && !isdigit(s[p])) p--;		// p -   
	if (p<0) return;							//  
	while (p >= 0 && isdigit(s[p]))
	{
		if (s[p]<'9') { s[p]++; break; }			// <9,    
		s[p--] = '0';							//   9,     0    
	}
}

///////////////////////////////////////////////////////////////////////
int		iCadr(char *s)	//      
{
	int res = 0, pow = 1;
	int p = (int)strlen(s) - 1;						// p -     
	while (p >= 0 && !isdigit(s[p])) p--;		// p -   
	while (p >= 0 && isdigit(s[p]))
	{
		res += pow*(s[p--] - '0');
		pow *= 10;
	}
	return res;
}
/////////////////////////////////////////////////////////////////////////
void	chExt(char *fileName, const char *ext)	//     ext
{
	char *pnt = strrchr(fileName, '.');
	if (pnt)	strcpy(pnt + 1, ext);
}

/////////////////////////////////////////////////////////////////////////
long long	getMillisec()		//  ,    ....
{
	struct _timeb timebuffer;
	_ftime(&timebuffer);
	long long sec = timebuffer.time;
	long long ms = timebuffer.millitm;
	return sec * 1000 + ms;
}
//-----------------------------------------------------------------------------
ldouble	getSec()		// number of seconds since ....
{
	struct _timeb timebuffer;
	_ftime(&timebuffer);
	//  int sec = (int)timebuffer.time % 1000000;
	ldouble sec = (ldouble)timebuffer.time;
	int ms = timebuffer.millitm;
	return sec + 0.001*ms;
}

//======================================================================
//==============      =========================
//======================================================================
#ifdef _WIN32
#include <Windows.h>
//     
i64 TimerCounter()
{
	i64 tm;
	QueryPerformanceCounter((LARGE_INTEGER *)(&tm));      //   
	return tm;
}
i64 TimerFrequency()
{
	i64 tm;
	QueryPerformanceFrequency((LARGE_INTEGER *)(&tm));    //   
	return tm;
}
#endif
/*
//  :
i64 tm = -TimerCounter();    //     

//  - 

tm  += TimerCounter();		//   ,     

//     (   ):
ldouble dtm=tm;
dtm /= TimerFrequency();

//  dtm -     

*/

//////////////////////////////////////////////////////////////////////
// -   
//  p=2^64-59 -   2 -     p.
//   1,2,4,8,... mod   p-1.
static u64 hRandom_value = 8446744073709551557L;
void    hRandomize(unsigned init)		//  init=0,  -  
{
	const u64 hRand_p = 18446744073709551557ULL;
	hRandom_value = init>0 ? init : 1234567891234567L * (i64)time(NULL);
	if (hRand_p < hRandom_value) hRandom_value -= hRand_p;
}

//u64 hRandom_next() {
//	const u64 p2 = 9223372036854775778L;
//
//	if (hRandom_value <= p2) hRandom_value *= 2;
//	else  hRandom_value = 2 * hRandom_value + 59;
//
//	//   :
//	u64 t = (hRandom_value * 0x12345678abcdefL) ^ 0x4937837937219836L;
//	return t;
//}
u64 hRandom_next() {
	const u64 p2 = 100000000000000003ll;		// p2 := nextprime(10^17)
	const unsigned a = 105;						// primroot(100,p2);

	hRandom_value = hRandom_value*a % p2;

	//   :
	u64 t = (hRandom_value + 0x12345678abcdefll) * 0x4937837937219836ll;
	return t;
}

int     hRandom(unsigned k)		//     0  k-1
{
	return hRandom_next() % k;
}

ldouble  hRand()	//    0  1
{
	const u64 u64max = 18446744073709551557ULL;	// 2^64-59
	const ldouble coeff = 1 / ((ldouble)u64max);
	return hRandom_next()*coeff;
}

//---------------------------------------------------------------------------------
ldouble  hRand_Std()	// ...  (m=0, s=1)
{
	ldouble res = 0;
	for (int i = 0; i<12; i++) res += hRand();
	return res - 6;
}
//---------------------------------------------------------------------------------
ldouble  hRand_Normal(ldouble m, ldouble s)	// ... .=m, ..=s
{
	return m + hRand_Std()*s;
}

//////////////////////////////////////////////////////////////////
static const int NPri = 32;
static const unsigned Prim_pow2[NPri] = {	//  ,  2^k
	1,  //   0
	3,  //   1
	5,  //   2
	11,  //   3
	17,  //   4
	37,  //   5
	67,  //   6
	131,  //   7
	257,  //   8
	521,  //   9
	1031,  //  10
	2053,  //  11
	4099,  //  12
	8209,  //  13
	16411,  //  14
	32771,  //  15
	65537,  //  16
	131101,  //  17
	262147,  //  18
	524309,  //  19
	1048583,  //  20
	2097169,  //  21
	4194319,  //  22
	8388617,  //  23
	16777259,  //  24
	33554467,  //  25
	67108879,  //  26
	134217757,  //  27
	268435459,  //  28
	536870923,  //  29
	1073741827,  //  30
	2147483659U,  //  31
};
static const unsigned Prim_base[NPri] = {	//     p[k]
	0, //   0
	2, //   1
	2, //   2
	6, //   3
	5, //   4
	13, //   5
	18, //   6
	37, //   7
	65, //   8
	134, //   9
	257, //  10
	516, //  11
	1032, //  12
	2049, //  13
	4098, //  14
	8196, //  15
	16385, //  16
	32777, //  17
	65538, //  18
	131073, //  19
	262145, //  20
	524304, //  21
	1048577, //  22
	2097153, //  23
	4194308, //  24
	8388611, //  25
	16777217, //  26
	33554435, //  27
	67108870, //  28
	134217732, //  29
	268435458, //  30
	536870920, //  31
};
//////////////////////////////////////////////////////////////////
unsigned Mnext(unsigned N, unsigned x)			//   0..N-1   
{
	int ip = ilog2(N)+1;
	unsigned pp = Prim_pow2[ip];
	unsigned aa = Prim_base[ip];
	u64 t = x + 1;
	do {
		t = (t*aa) % pp;
	} while (t > N);
	x = (unsigned)t;
	return x - 1;
}
//=============================================================================
//  size   src   stepSrc  dst   stepDst
void	copyBuf(int *dst, int stepDst, int *src, int stepSrc, int size)
{
	while (size--) { *dst = *src; dst += stepDst; src += stepSrc; }
}

/////////////////////////////////////////////////////
//      
//
//           |                        |
//           |				         182
//      -----+----> x		     ----705---> x
//           |				         463
//           | y			          | y
//           v				          v
//
//   0)  0  0
//   1)  1 -1
//   2)  1  1
//   3) -1  1
//   4) -1 -1
//   5)  1  0
//   6)  0  1
//   7) -1  0
//   8)  0 -1
//   9)  2 -2
//  10)  2  2
//
void	nextPoint(int &x, int &y) {
	//    90    
	int t = y; y = x; x = -t;
	if (x<abs(y)) return;
	if (x>0 && x == y) return;
	if (y<x - 1) { y++; return; }
	x++; y = -x;
}
//////////////////////////////////////////////////////////////////////
void	rgb_yuv( int  r, int  g, int  b, int &y, int &u, int &v )
{
	y = ( 19595*r +38470*g + 7471*b         )>>16;
	u = (-11058*r -21710*g +32768*b +8388608)>>16;
	v = ( 32768*r -27439*g - 5329*b +8388608)>>16;
}
void	yuv_rgb( int &r, int &g, int &b, int  y, int  u, int  v )
{
	u -= 128; v -=128;
	r = (65536*y         + 91881*v)>>16;
	g = (65536*y- 22553*u- 46802*v)>>16;
	b = (65536*y+116130*u         )>>16;
}

int rgb_int( int r, int g, int b ) {  // (r,g,b) -> x
	r = r<=0 ? 0 : r>=255 ? 255 : r;
	g = g<=0 ? 0 : g>=255 ? 255 : g;
	b = b<=0 ? 0 : b>=255 ? 255 : b;
	return r + (g<<8) + (b<<16);
}

void int_rgb( int x, int &r, int &g, int &b) { // x -> (r,g,b)
	r =  x      & 255;
	g = (x>> 8) & 255;
	b = (x>>16) & 255;
}
///////////////////////////////////////////////////////////////////////////
//  
void length1( int &x, int &y)
{
	int signX = (x>=0 ? 1 : -1 ); x*=signX;
	int signY = (y>=0 ? 1 : -1 ); y*=signY;
	if( x>=2*y )      { x=1; y=0; }
	else if( y>=2*x ) { x=0; y=1; }
	else              { x=1; y=1; }

	x *=signX;
	y *=signY;
}

/////////////////////////////////////////////////////////////////////////////
//
//              3
//            2 * 0
//              1
//
void neighbor(int x, int y, int k, int &x1, int &y1, int h)	//    k=0|1|2|3   h
{
	x1 = x; y1 = y;
	if (k == 0) x1+=h;
	else if (k == 1) y1+= h;
	else if (k == 2) x1-= h;
	else          y1-= h;
}

void neighbor8(int x, int y, int k, int &x1, int &y1, int h)	//    k=0..7   h
{
	// 734
	// 2*0
	// 615

	x1 = x; y1 = y;
	if (k == 0) x1+= h;
	else if (k == 1) y1+= h;
	else if (k == 2) x1-= h;
	else if (k == 3) y1-= h;
	else if (k == 4) { x1+= h; y1-= h; }
	else if (k == 5) { x1+= h; y1+= h; }
	else if (k == 6) { x1-= h; y1+= h; }
	else { x1-= h; y1-= h; }
}



//-----------------------------------------------------------------------------
bool boxBord(int x0, int y0, int dx, int dy, int x1, int y1)	//    <x1,y1>   
{
	if (y1 == y0 || (y1 == y0 + dy)) {		// / 
		if (x0 <= x1 && x1 <= x0 + dx) return true;
		return false;
	}
	if (x1 == x0 || (x1 == x0 + dx)) {		// / 
		if (y0 <= y1 && y1 <= y0 + dy) return true;
		return false;
	}
	return false;
}
//-----------------------------------------------------------------------------
// ,  dx>0, dy>0
void boxNext(int x0, int y0, int dx, int dy, int &x1, int &y1)	// <x1,y1> -      
{
	if (!boxBord(x0, y0, dx, dy, x1, y1)) return; //   (x1,y1)     ,    
	if (y1 == y0) {		//  
		if (x1 < x0 + dx)  x1++; else y1++;
		return;
	}
	if (x1 == x0 + dx) {	//  
		if (y1 < y0 + dy)  y1++; else x1--;
		return;
	}
	if (y1 == y0 + dy) {//  
		if (x1 > x0)  x1--; else y1--;
		return;
	}
	if (x1 == x0) {		//  
		if (y1 > y0)  y1--; else x1++;
		return;
	}
}
//-----------------------------------------------------------------------------
// ,  dx>0, dy>0
//   (x1,y1)     ,    
void boxPrev(int x0, int y0, int dx, int dy, int &x1, int &y1)	// <x1,y1> -      
{
	if (!boxBord(x0, y0, dx, dy, x1, y1)) return; //   (x1,y1)     ,    
	if (y1 == y0) {		//  
		if (x1 > x0)  x1--; else y1++;
		return;
	}
	if (x1 == x0 + dx) {	//  
		if (y1 > y0)  y1--; else x1--;
		return;
	}
	if (y1 == y0 + dy) {//  
		if (x1 < x0 + dx)  x1++; else y1--;
		return;
	}
	if (x1 == x0) {		//  
		if (y1 < y0 + dy)  y1++; else x1++;
		return;
	}
}



/////////////////////////////////////////////////////////////////////////////////////

